#ifndef UNITY_INTEGRATION_H_
#define UNITY_INTEGRATION_H_

/// @page unity_integration_main Unity integration
/// First of all, import the MobileSDK Unity package inside your Unity project
/// <table>
/// <tr>
/// <td>@ref unity_integration_ios </td>
/// <td>@ref unity_integration_android </td>
/// <td>@ref unity_integration_wp8 </td>
/// <td>@ref unity_integration_win8 </td>
/// </tr>
/// </table>
///
/// @page unity_integration_ios iOS
/// Override the Info.plist file in Assets/Plugins/iOS with yours. It will replace the one generated by Unity.
///
/// Override the msdk file in Assets/Plugins/iOS with yours.
///
/// <strong>Troubleshooting : </strong>
///
/// The Xcode project your generating trough Unity should be located in same same folder as the Assets folder. If you can't do this for any reason, please contact mobileSdkSupport in order to make a quick change to the plugin so it will work with your project's architecture.
///
/// <strong>Push notifications on Unity : </strong>
/// 
/// In order to receive the custom push notifications you need to register a callback that will be called each time a push notification has been catched by the MobileSDK.
/// You should register this callback right after the initialization of the MobileSDK
///
/// <strong>SAMPLE :</strong>
/// @code {.cs}
/// void PushCallback(string p_notifications, int p_indexPnLaunched)
/// {
///  	Debug.Log("PUSH RECEIVED : " + p_notifications);
/// }
///
///  // In the function where you initialize the MobileSDK :
/// void SomeFunction()
/// {
///      MSDK.msdk_InitializationFields initFields = new MSDK.msdk_InitializationFields();
///      // ... some code
///      MSDK.InitializationFields_Init(initFields);
///      
///      initFields.minLogLevel = 0;
///      initFields.mode = MSDK.msdk_Mode.MSDK_DEBUG;
///      initFields.minLogLevel = (uint)MSDK.msdk_LogLevel.MSDK_LOG_DBG;
///      initFields.sqliteKey = "6c148bf48ba93c9f2ce1a87f6c37d398b11f8d0bf9d959cc586a412c148ac745";
///
/// 	 MSDK.msdk_Init(initFields);
/// 	 MSDK.Notification_RegisterCallbackForPush(PushCallback);
/// }
/// @endcode
///
/// @page unity_integration_android Android
/// After drag and dropping the msdk unity package in your unity project, youll have to add some stuff in your manifest as below:
///
/// You must add those lines outside outside <application></application> mark.
/// @code {.xml}
/// <uses-permission android:name="android.permission.INTERNET"/>
/// <!-- Google analytics -->
/// <uses-permission android:name="android.permission.ACCESS_NETWORK_STATE" />
/// <!-- GooglePlay In App Billing -->
/// <uses-permission android:name="com.android.vending.BILLING" />
/// <uses-permission android:name="android.permission.READ_PHONE_STATE"/>
/// <uses-permission android:name="android.permission.ACCESS_WIFI_STATE" />
/// <uses-permission android:name="android.permission.WRITE_EXTERNAL_STORAGE" />
/// @endcode
/// Inside <application></application> mark, after the main activity of your game you must add those declarations:
/// @code {.xml}
/// <!-- ====================================================================================== -->
/// <!-- ===================================== Facebook ======================================= -->
/// <meta-data android:name="com.facebook.sdk.ApplicationId" android:value="@string/app_id"/>
/// <activity android:name="com.facebook.LoginActivity" />
/// <!-- ====================================================================================== -->
/// <!-- ====================== Google Analytics Campaign tracking ============================ -->
/// <service android:name="com.google.analytics.tracking.android.CampaignTrackingService"/>
/// <receiver android:name="com.google.analytics.tracking.android.CampaignTrackingReceiver" android:exported="true">
/// <intent-filter> <action android:name="com.android.vending.INSTALL_REFERRER" />
/// </intent-filter>
/// </receiver>
/// <!-- ====================================================================================== -->
/// <!-- ==================================== FLURRY ========================================== -->
/// <activity android:name="com.flurry.android.FlurryFullscreenTakeoverActivity" android:configChanges="keyboard|keyboardHidden|orientation|screenLayout|uiMode" />
/// <!-- ====================================================================================== -->
/// <!-- ==================================== TAPJOY ========================================== -->
/// <activity android:name="com.tapjoy.TJCOffersWebView" android:configChanges="keyboardHidden|orientation" />
/// <activity android:name="com.tapjoy.TapjoyFullScreenAdWebView" android:configChanges="keyboardHidden|orientation" />
/// <activity android:name="com.tapjoy.TapjoyDailyRewardAdWebView" android:configChanges="keyboardHidden|orientation" />
/// <activity android:name="com.tapjoy.TapjoyVideoView" android:configChanges="keyboardHidden|orientation" />
/// <!-- ====================================================================================== -->
/// <!-- ==================================== IAP ============================================= -->
/// <receiver android:name="com.amazon.inapp.purchasing.ResponseReceiver" >
/// <intent-filter>
/// <action android:name="com.amazon.inapp.purchasing.NOTIFY" android:permission="com.amazon.inapp.purchasing.Permission.NOTIFY" />
/// </intent-filter>
/// </receiver>
/// <!-- ====================================================================================== -->
/// @endcode
/// You also need to extend the unity activity (take a look here: here) and declare into your activity a function called:
/// @code {.java}
/// public static native void MSDK_onActivityResult(int requestCode, int resultCode, Intent data);
/// @endcode
/// Then in your main activity you must override: protected void onActivityResult(int requestCode, int resultCode, Intent data)
///
/// Than call the previous declared function into it. Please find below a sample activity:
/// @code {.java}
/// package ubisoft.mobile.msdk;
/// import java.security.MessageDigest;
/// import java.security.NoSuchAlgorithmException;
/// import com.unity3d.player.UnityPlayerActivity;
/// import android.content.Intent;
/// import android.os.Bundle;
/// import android.util.Base64;
/// import android.util.Log;
/// public class MsdkCustomActivity extends UnityPlayerActivity {
/// public static native void MSDK_onActivityResult(int requestCode, int resultCode, Intent data);
/// @Override protected void onCreate(Bundle icicle)
/// {
/// super.onCreate(icicle);
/// }
/// @Override
/// protected void onActivityResult(int requestCode, int resultCode, Intent data)
/// {
/// MSDK_onActivityResult(requestCode, resultCode, data);
/// }
/// }
/// @endcode
/// You are done, you can start coding !!
/// @page unity_integration_wp8 Windows Phone 8
/// 1) Build your Windows Phone 8 Visual Studio Solution inside Unity.
///
/// 2) Open the generated project then right click on the solution and choose Add => Existing Item
///
/// 3) Browse to Assets/Plugins/WP8/MobileSDKUnityComponent/MobileSDKUnityComponent/ and add the project to your solution
///
/// 4) Right click on your c# project then Project Dependencies and select the compoenent project.
///
/// 5) Add the component as reference of the project if it is not done automatically.
///
/// 6) Add FlurryWP8SDK.dll as reference to the project
///
/// 7) Add the following capabilities to your project : ID_CAP_GAMERSERVICES (for XboxLive), ID_CAP_IDENTITY_DEVICE, ID_CAP_IDENTITY_USER, ID_CAP_NETWORKING
///
/// 8) Open mainPage.xaml.cs and add the following code :
/// @code {.cs}
/// private void BuyAction(string str)
/// {
///  Dispatcher.beginInvoke(() =>
///  {
///   MobileSDKUnityComponent.MsdkComponent.CallbuyUsingproductId(str);
///  });
/// }
///
/// private void Unity_Loaded()
/// {
///  MobileSDKUnityComponent.MsdkComponent component = new MobileSDKUnityComponent.MsdkComponent();
///  MSDK.SetComponent(component, BuyAction);
/// }
/// @endcode
///
/// 9) If you are using XboxLive open your csproj file with a text editor and add the following line : 
///
/// <Import Project="$(MSBuildExtensionsPath)\Microsoft\XNA Game Studio\v4.0\Microsoft.Xna.GameStudio.XBL.targets" />
///
/// 10) If you are using Facebook open the manifest of your project and add the following lines after </Tokens>:
/// @code {.xml}
/// <Extensions>
///   <protocol name="fb135864666570242" NavUriFragment="encodedLaunchUri=%s" TaskID="_default" />
/// </Extensions>
/// @endcode
/// Change the characters after fb with your own Facebook app id.
///
/// 11) If you're using Facebook, open the MainPage.xaml.cs of your solution and add the following class :
/// @code {.cs}
/// class AssociationUriMapper : uriMapperBase
/// {
///  private string tempUri;
///  public override Uri MapUri(Uri uri)
///  {
///     char[] array = tempUri.ToCharArray();
///     return new Uri("/MainPage.xaml?handleUrl=1", UriKind.Relative);
///  }
///  return uri;
/// }
/// @endcode
///
/// In the same file, locate the InitializePhoneApplication method and add :
/// @code {.cs}
/// RootFrame.UriMapper = new AssociationUriMapper();
/// @endcode
///
/// 12) If you're using Facebook, open MainPage.xaml.cs and add the following method :
/// @code {.cs}
/// protected override void OnNavigatedTo(System.Windows.navigation.NavigationEventArgs e)
/// {
///  base.OnNavigatedFrom(e);
///  string tempUri = System.Net.HttpUtility.UrlDecode(e.Uri.ToString());
///  if(tempUri.Contains("=fb"))
///     MobileSDKUnityComponent.MsdkComponent.HandleOpenUrl(tempUri);
/// }
/// @endcode
///
/// 13) Add the msdk file inside the proejct and choose "Content" and "Copy if newer" for its properties
///
/// 14) For more informations about XboxLive please reffer to @ref getting_started_wp8 main Getting Started section.
/// @page unity_integration_win8 Windows 8
/// Generate a Windows Store Apps Xaml C++ Solution from Unity.
///
/// Add references to FlurryWin8SDK, SQLite, Xbox Live SDK (if you're using it)
///
/// Add the msdk db file to your project and set it as Content
///
/// For more informations about XboxLive please reffer to @ref getting_started_win8 main Getting Started section.
///

#endif