#ifndef MOBILE_SDK_API_AD_H_
# define MOBILE_SDK_API_AD_H_

#include "MobileSDK/common/Utils.h"
#include "MobileSDK/ad/AdUtils.h"
#include "MobileSDK/common/Init.h"
#include "MobileSDK/ad/AdInterface.h"

#ifdef __cplusplus
extern "C"
{
#endif
    ///
	/// <strong>Must be called before Ad_CallInit.</strong>
	/// Use to register an ad provider not included in mobile sdk by default.
	/// Currently mobile SDK include as Ad Provider : Flurry and Tapjoy.
	/// If you need to add a new Ad Provider (like AdMob, Trial Play, iAd, ...) you can do so using that function.
	/// For more information please take a look at msdk_AdInterface documentation
	/// Once added you'll be able to configure from server side you new ad provider as any default ad provider included in msdk.
	/// @param p_interface A new ad interface to add to the mobile SDK.
	///
	void						Ad_AddProvider			(msdk_AdInterface* p_interface);

    ///
    /// Configure a currency. It sets the currency id specific to a network to use for the generic currencyName specified during the preload
    /// @param p_currencyName The generic name of the currency across the services
    /// @param p_service The name of the service to configure the currency for. (embedded <strong>case sensitive</strong> services name are tapjoy, flurry, Trialpay)
    /// @param p_currencyId The id of the currency on the given service
    ///
    void                        Ad_ConfigureCurrency(const char_utf8* p_currencyName, const char_utf8* p_service, const char_utf8* p_currencyId);

	///
	/// Initialize the ads. <strong>Its compulsory before making any other request to ad API </strong> except Ad_AddProvider.
	/// @return the id of the request if successfully started, -1 otherwise.
	///
	msdk_s8						Ad_CallInit				();

	///
	/// Get the status of the request for initializing ads.
	/// @param p_id Id of the init request (see Ad_CallInit)
	/// @return The status of the request (MSDK_ENDED if the request is finished)
	///
	msdk_Status					Ad_StatusInit			(msdk_s8 p_id);

	///
	/// Get the result of the ad initialization. Must be called after Ad_StatusInit(p_requestid) return MSDK_ENDED.
	/// @param p_id Id of the init request (see Ad_CallInit)
	/// @return The result of the ads initialization.
	///
	msdk_Result					Ad_ResultInit			(msdk_s8 p_id);

	///
	/// <strong>Must</strong> be call after Ad_ResultInit. It release the internal memory used during ad initialization. It does not uninit ads.
	/// @param p_id Id of the init request (see Ad_CallInit)
	///
	void						Ad_ReleaseInit			(msdk_s8 p_id);

	///
	/// Start pre-loading an ad. Once this request is ended (and result successful) you'll be able to display an ad without additional loading time.
	/// @param p_adInfo An msdk_AdInfo structure that give information regarding type of ad (Fullscreen, banner, offerwall, etc) and a view to display it.
	/// @return the id of that request if successfully started, -1 otherwise.
	///
	msdk_s8 					Ad_CallPreload			(msdk_AdInfo* p_adInfo);

	///
	///	Get the status of the pre-loading request.
	/// @param p_id The id of the request
	///	@return The status of the request
	///
	msdk_Status 				Ad_StatusPreload		(msdk_s8 p_id);

	///
	///	Get the result of the pre-loading request.
	/// @param p_id The id of the request
	///	@return The result of the request
	/// |     Value	       |    Description   |
	/// | :--------------: | :--------------: |
	/// | MSDK_SUCCESS     | The ad has preloaded successfully |
	/// | MSDK_NETWORK_ERROR | An error occured while preloading the ad. Check the internet connection and your provider id in the msdk db |
	/// | MSDK_UNKNOWN_ERROR | Bad request id |
	///
	msdk_Result					Ad_ResultPreload		(msdk_s8 p_id);

	///
	///	Display a preloaded ad on the screen
	/// @param p_id The id of the ad you have preloaded before
	///
	void 						Ad_Display				(msdk_s8 p_id);

	///
	///	Change the visibility of a displayed ad on the screen
	/// @param p_id The id of the ad
	/// @param p_visibility MSDK_TRUE if you want the ad to be visible, MSDK_FALSE if you want to hide it
	///
	void 						Ad_Visibility			(msdk_s8 p_id, char p_visibility);

	///
	///	Pop the last catched events of an ad
	/// @param p_id The id of the ad
	/// @return An array which contains the last events of the ad. The events got are popped from the list of events
	///
	Array_msdk_AdEvent*			Ad_Events				(msdk_s8 p_id);

	///
	///	Delete an ad. (if it is displayed it will stop display)
	/// @param p_id The id of the ad
	///
	void						Ad_Delete				(msdk_s8 p_id);

	///
	///	Make a request to check if the user has got some virtual currency.
	///	Once a request is performed, the amount retrieved will not be available again from the server. You have to store it inside your game.
	/// @return The id of the request
	///
	msdk_s8						Ad_CallVirtualCurrency	();

	///
	///	Get the status of a CallVirtualCurrency request
	/// @return The status of the request
	///
	msdk_Status					Ad_StatusVirtualCurrency(msdk_s8 p_id);

	///
	///	Returns the result of the virtual currency request
	/// @param p_id The id of the request
	/// @return An array containing the currency earned by the player for each currency.
	///
	Array_msdk_VirtualCurrency*	Ad_ResultVirtualCurrency(msdk_s8 p_id);

	///
	///	Release the memory allocated for the virtual currency request
	/// @param p_id The id of the request
	///
	void						Ad_ReleaseVirtualCurrency(msdk_s8 p_id);
#ifdef __cplusplus
}
#endif



#endif // MOBILE_SDK_API_AD_H_
