#ifndef GETTING_STARTED_IOS
#define GETTING_STARTED_IOS

/// @page getting_started_wp8 wp8
///
/// @section wp8_settings Project settings
/// In order to be able to use Facebook and Flurry you need to create a Direct3DXamlApp project.
/// You have to make the following changes to the c++ component of your application.
/// You need to link with several libraries (inside properties => Linker => Input) :
/// <ul>
/// <li>MobileSDK.lib (the MobileSDK)</li>
/// <li>msxml6.lib (used to perform HTTP requests)</li>
/// </ul>
///
/// You also need to add the path to the MobileSDK.lib file inside the "Additional Library Directories" section (Linker => General => Additional Library Directories)
/// In order to include the MobileSDK functions you need add the path to the MobileSDK headers (C/C++ => General => Additional Include Directories)
/// 
/// You should define the preprocessor macro : UBI_WP8
/// Finally, you need to add several references (Common Properties => Framework and References) :
/// <ul>
/// <li>FlurryWin8SDK (can be found in the libraries folder of the package)</li>
/// <li>SQLite.WP80 (see @ref wp8_download_sqlite_extension)</li>
/// <li>Xbox LIVE SDK for Windows Phone (see @ref wp8_xbox_live_setup)</li>
/// </ul>
///
/// Some capabilities are required by the MobileSDK to work properly. You should activate inside the manifest of your application :
/// <ul>
/// <li>ID_CAP_GAMERSERVICES (only if you use XboxLive)</li>
/// <li>ID_CAP_IDENTITY_DEVICE</li>
/// <li>ID_CAP_IDENTITY_USER</li>
/// <li>ID_CAP_NETWORKING</li>
/// </ul>
///
/// @section wp8_flurry_setup Setup Flurry
/// In order to be able to use Flurry you need to make few changes to project.
/// This is needed because FlurrySDK is only available in c# so we need to make a bridge from c++ to c#.
/// First of all, copy the inl files located in the misc/wp8 folder of the MobileSDK package and add them to your c++ component (MsdkBridge.inl and MsdkInterface.inl).
/// You can now take the FlurryInterfaceImpl.cs file and add it to the c# project of your solution.
/// Open this file and change the namespaces used by your own namespaces.
///
/// You need to modify the c++ component. Here is an example of a modified component (the added lines of code are commented)
/// Changes in the header :
/// @code {.cpp}
/// namespace PhoneDirect3DXamlAppComponent
/// {
/// #define XAML_COMPONENT PhoneDirect3DXamlAppComponent //Needed be cause we don't know the package name in the .inl files
/// #include "MsdkInterface.inl"  //needed to references the functions in the c#
/// [Windows::Foundation::Metadata::WebHostHidden]
/// public ref class Direct3DBaclground sealed : public ...
/// {
/// public:
/// Direct3DBackground();
/// Direct3DBackground(ICallback^ callback); // We need to overload the constructor so we will pass the Flurry interface from the c# to the c++ then we will call the c# from the c++
/// ...
/// };
/// }
/// @endcode
/// Changes in the implementation
/// @code {.cpp}
/// PhoneDirect3DXamlAppComponent::ICallback^ msdkCallback = nullptr; //will be used to store the flurry interface given from the c#. Use the namespace of your component here
/// namespace PhoneDirect3DXamlAppComponent
/// {
/// Direct3DBackground::Direct3DBackground(ICallback^ callback) :
///  m_timer(ref new basicTimer())
/// {
///    msdkCallback = callback; //set the flurry callback to the one given by the c#
///    ... // your code
/// }
/// ... // the implementation of your component
/// } // closure of the namespace of your component
/// #include "MsdkBridge.inl" //Declares the functions that will be called by the MobileSDK so it can call the C#
/// }
/// @endcode
///
/// Now that the c++ component is setup, we can modify the C# project where we'll create the object that will call the FlurrySDK and give it to the c++ component so we can consume it inside the MobileSDK.
/// In your MainPage.xaml.cs file you should have a DrawingSurfaceBackground_Loaded method. We'll perform a few changes in it (changes are commented)
/// @code {.cs}
/// private void DrawingSurfaceBackground_Loaded(object sender, RoutedEventArgs e)
/// {
///  if(m_d3dBackground == null)
///  {
///     MsdkInterfaceImpl mii = new MsdkInterfaceImpl(); //we create the object that will call the FlurrySDK
///		m_d3dBackground = new Direct3DBackground(mii); // we pass the created object to the c++ so we can call it from the MobileSDK
///  }
/// }
/// @endcode
///
/// @section wp8_download_sqlite_extension Install the SQLite extension for WIndows Phone
/// The MobileSDK library uses SQLite to store datas. In order to reference SQLite in your project you need to install it on your computer before.
/// Here is a link to the download page of the package : <a href="http://www.sqlite.org/download.html">Sqlite</a>
/// You need to look for Precompiled Binaries for Windows Phone 8, download the file and install it. Reboot Visual Studio and you should be able to reference SQLite in your project (under Windows Phone => Extensions).
///
/// @section wp8_xbox_live_setup XboxLive setup for WindowsPhone 8
/// In order to develop for XboxLive on Windows Phone 8 several things are required :
/// <ul>
/// <li>Xbox Live SDK for Windows Phone 8 (will allow you to add a reference to XboxLiveSDK in your project)</li>
/// <li>Xbox Live Publishing Toolkit (allows to manage the achievements, leaderboard, etc.)</li>
/// <li>PartnerNet RegKey (will add an entry to the Windows's registries)</li>
/// <li>WP8 CertStamper (needed to setup your device to connect the dev environment of XboxLive)</li>
/// <li>A .pfx file which is the certificate</li>
/// <li>Certification password (needed to setup your device to connect the dev environment of XboxLive)</li>
/// <li>A Microsoft developer account</li>
/// <li>An XboxLive developer account linked with your Microsft account and your titleId</li>
/// </ul>
/// Make a request to Digital Certification to get these tools.
///
/// Once you have all these tools you can connect to XboxLive inside your app by following these steps :
///
/// 1) Install Xbox Live SDK for Windows Phone 8 (and add a reference to it in your c++ component)
///
/// 2) Install the Xbox Live Publishing Toolkit (the .exe will be located in C:\Program Files (x86)\Xbox LIVE Publishing Toolkit)
///
/// 3) Double click on the PartnerNet file and accept.
///
/// 4) Reset your device (Settings => About => Reset your phone)
///
/// 4) Connect your Windows Phone 8 device
///
/// 5) Unlock your device for develoment using the Windows Phone Developer Registration tool (should be installed by default on your Windows 8)
///
/// 6) Find the folder that contains the Windows phone 8 CertStamper.exe and the associated files.
///
/// 7) Hold Shift key and right-click the folder then click <strong>Open command windows here</strong>
///
/// 8) Type Certstamper.exe PFXFilename PFXPassword
///
/// 9) Follow the instructions and your device should be able to use XboxLive for development
///
/// 10) Open the XboxLivePublishingToolkit and create the spa file to include in your project (open the XboxLive Publishing Toolkit for more informations)
///
/// 11) Open your .csproj file and add the following line : <Import Project="$(MSBuildExtensionsPath)\Microsoft\XNA Game Studio\v4.0\Microsoft.Xna.GameStudio.XBL.targets" />
///
/// 12) Tick the ID_CAP_GAMERSERVICES capability inside the manifest
///
/// 13) Set the build action for xbl.spa to LiveGameConfig
///
/// For more informations please reffer directly to the Microsoft documentation.
///
/// Troubleshooting :
///
/// If somehow you're not able to link your microsoft test account on the device, create a test account on live-int.com and use it.
///
/// @section wp8_facebook_setup Configuring your app to use Facebook
/// On Windows Phone 8 the MobileSDK redirect the user to Internet Explorer to login into Facebook.
///
/// In order to allow the user to go back to your application after he logged inside Facebook you need to perform a few changes inside your application :
///
/// 1) Open the WMAppManifest.xml fiel with the XML (Text) Editor
///
/// 2) Add the folowing lines after </Tokens>
/// @code {.xml}
/// <Extensions>
///   <Protocol Name="fb135864666570242" NavUriFragment="encodedLaunchUri=%s" TaskID="_default" />
/// </Extensions>
/// @endcode
/// In the name field the characters following fb should be replaced by your own Facebook app id.
/// 3) Open the App.xaml.cs file and add the following class :
/// @code {.cs}
/// class AssociationUriMapper : uriMapperBase
/// {
///  private string tempUri;
///  public override Uri MapUri(Uri uri)
///  {
///     char[] array = tempUri.ToCharArray();
///     return new Uri("/MainPage.xaml?handleUrl=1", UriKind.Relative);
///  }
///  return uri;
/// }
/// @endcode
/// 4) In the same file, locate the InitializePhoneApplication method and add :
/// @code {.cs}
/// RootFrame.UriMapper = new AssociationUriMapper();
/// @endcode
/// 5) Open the header of your c++ component and add the following declaration inside the public section of the class :
/// @code {.cpp}
/// void SocialConnection_OpenUrl(Platform::String^ p_url);
/// @endcode
/// 6) Open the .cpp file of your component and add this implementation of the method :
/// @code
/// void Direct3DBackground::SocialConnection_OpenUrl(Platform::String^ p_url)
/// {
///  msdk_HandleOpenURL(p_url->Data());
/// }
/// @endcode
/// 7) Open the MainPage.xaml.cs file of the c# project
///
/// 8) Add the following method to the Mainpage class :
/// @code {.cs}
/// protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
/// {
///  base.OnNavigatedFrom(e);
///  string tempUri = System.Net.HttpUtility.UrlDecode(e.Uri.ToString());
///  if(tempUri.Contains("=fb"))
///		m_uri = tempUri; //declare m_uri as a member of the class
/// }
/// @endcode
/// 9) Add the following lines inside the DrawingSurfaceBackground_Loaded method
/// @code {.cs}
/// if(m_uri == null)
///    m_d3dBackground.SocialConnection_OpenUrl(m_uri); // add thse lines after the m_d3dBackground creation
/// @endcode
/// You're app should now be able to be called back from Internet Explorer.
///
/// @section wp8_initialize_msdk Setup the msdk file
/// The MobileSDK uses a configuration file(msdk) that you need to include inside your application.
///
/// You can retreive this file which is specific to your project on the <a href="https://ubimobile-admin.ubisoft.org/ubimobiadm_dev/index.php/">Houston</a> server.
///
/// Add this file to your c# project. Click on it. in the properties tab under Build Action choose Content. Under the Copy to Output field choose Copy if newer
///

#endif

