#ifndef MOBILESDK_DYNAMIC_STORE_H_
#define MOBILESDK_DYNAMIC_STORE_H_

#include "MobileSDK/common/Utils.h"
#include "MobileSDK/iab/IAPManager.h"

#ifdef __cplusplus
extern "C"
{
#endif
    ///
	/// A game item
	///
    typedef struct msdk_StoreItem    {
        msdk_u32                itemId;         //!< The id of the item
        const char_utf8*        title;          //!< The name of the item
        msdk_ProductType        type;           //!< The type of the item
        const char_utf8*        description;    //!< The description of the item
        Array_msdk_key_value*   args;           //!< The args of the item
        msdk_u32                amount;         //!< The amount of the item (only filled after Store_GetUserItems or Store_BuySecondaryStore)
    }msdk_StoreItem;

    DECLARE_ARRAYS(msdk_u32);

    ///
	/// A promotion
	///
    typedef struct msdk_StorePromo
    {
        msdk_u32            promoId;            //!< The id of the promotion
        const char_utf8*    startDate;          //!< The start date of the promotion
        const char_utf8*    endDate;            //!< The end date of the promotion
        const char_utf8*    description;        //!< The description of the promotion
        msdk_u32            percentageRebate;   //!< The percentage of rebate
        msdk_u32            percentageBonus;    //!< The percentage of bonus
        /*#ifdef MSDK_BUILD_LIB
            Array_msdk_u32*     affectedTrades;
        #else
            void*               internal;           //!< For internal use only
        #endif*/
        Array_msdk_u32*     affectedTrades;               //!< For internal use only
    }msdk_StorePromo;

    DECLARE_ARRAYS(msdk_StorePromo);
    DECLARE_ARRAYS(msdk_StoreItem);

    ///
	/// A primary store item
	///
    typedef struct msdk_PrimaryStoreItem
    {
        msdk_u32                  itemId;       //!< The id of the item
        msdk_IAPProduct*          storePack;    //!< The id of the pack
        Array_msdk_StoreItem*     gettings;     //!< The game's items the user will earned by buying this pack
        const char_utf8*          minVersion;   //!< The minimum version
        const char_utf8*          icon;         //!< The icon of the item
        PointerArray_char_utf8*   categoryTags; //!< The tags associated with the item
        Array_msdk_StorePromo*    promos;       //!< The promotions  associated with the item
        msdk_ProductType		  type;       	//!< Type of the trade Managed / Consumable / Subscription
        Array_msdk_key_value*     args;         //<! Args associated with the item (key is the number of the arg)
    }msdk_PrimaryStoreItem;
    
    ///
	/// The result of a purchase with the primary store
	///
    typedef struct msdk_PrimaryPurchaseResult
    {
        msdk_TransactionStatus	transactionStatus;  //!< The status of the transaction
        msdk_PrimaryStoreItem*  item;               //!< The item which was bought
    }msdk_PrimaryPurchaseResult;

    ///
	/// A secondary store item
	///
    typedef struct msdk_SecondaryStoreItem
    {
        msdk_u32                  itemId;           //!< The id of the item
        const char_utf8*          name;             //!< The name of the item
        const char_utf8*          description;      //!< The description of the item
        msdk_StoreItem*           priceItem;        //!< The item to spend to buy this one
        msdk_u32                  priceAmount;      //!< The amount of item to spend to buy this one
        Array_msdk_StoreItem*     gettings;         //!< The list of items the user will earn after buying this one
        const char_utf8*          minVersion;       //!< The minimum version
        const char_utf8*          icon;             //!< The icon of the item
        PointerArray_char_utf8*   categoryTags;     //!< The tags associated with the item
        Array_msdk_StorePromo*    promos;           //!< The promotions associated with the item
        Array_msdk_key_value*     args;             //<! Args associated with the item (key is the number of the arg)
    }msdk_SecondaryStoreItem;

    DECLARE_ARRAYS(msdk_PrimaryStoreItem);
    DECLARE_ARRAYS(msdk_SecondaryStoreItem);

    typedef void (*Store_RefreshCallback)(msdk_s8 p_isIapAvailable);
    
    ///
    /// Activate/Unactivate the auto refreshing of items during the resu;e of the app
    ///
    MSDK_EXPORT void                                Store_ActivateAutoRefresh(msdk_s8 p_isActivated);
    
    ///
    /// Activate/Unactivate the secondary storecloud save for durable items
    ///
    MSDK_EXPORT void Store_ActivateCloud(msdk_s8 p_isActivated);
    ///
    /// Initialize the store.
    /// @return The status of the request
    ///
    MSDK_EXPORT void                                Store_CallInit();

    ///
    /// Get the status of the initilization of the store
    /// @return The status of the initialization
    ///
	MSDK_EXPORT msdk_Status                         Store_StatusInit();

    ///
    /// Get the result of the initialization of the store
    /// @return The result of the initialization
    ///
	MSDK_EXPORT msdk_Result                         Store_ResultInit();

    ///
    /// Release the store, free all memory allocated.
    ///
	MSDK_EXPORT void                                Store_Release();

    ///
    /// Get the primary store
    /// @return The list of all the primary store items
    ///
	MSDK_EXPORT Array_msdk_PrimaryStoreItem*        Store_GetPrimaryStore();

    ///
    /// Get the secondary store
    /// @return The list of the secondary store items
    ///
	MSDK_EXPORT Array_msdk_SecondaryStoreItem*      Store_GetSecondaryStore();

    ///
    /// Get items of the game
    /// @return The list of the game's items
    ///
	MSDK_EXPORT Array_msdk_StoreItem*               Store_GetGameItems();

    ///
    /// Get the items of the user
    /// @return The list of the user's items
    ///
	MSDK_EXPORT Array_msdk_StoreItem*               Store_GetUserItems();

    ///
    /// Reset all the user items
    ///
	MSDK_EXPORT void                                Store_ResetUserItems();

    ///
    /// Export the items of the user (useful if you want to handle several saves for a player)
    /// @return An encrypted export of the user items
    ///
	MSDK_EXPORT msdk_FileContent                    Store_ExportUserItems();

    ///
    /// Import items for the users. Will erase all the existing items.
    /// @param p_dump An export of the user items previously generated with Sotre_UserItems
    /// @param p_size The size of the encrypted dump
    /// @return MSDK_TRUE on success, MSDK_FALSE otherwise
    ///
	MSDK_EXPORT msdk_s8                                Store_ImportUserItems(const char_utf8* p_dump, msdk_u32 p_size);

    ///
    /// Get number of a specific item owned by the user
    /// @param  p_item The item
    /// @return The amount of the item owned by the user
    ///
	MSDK_EXPORT msdk_u32                            Store_GetItemAmount(msdk_StoreItem* p_item);

    ///
    /// Credit an item for the user. This function should be called when the user win an item ingame without spending money
    /// The amount field must be specified in order to give the right amount to the user
    /// @param  p_item The item to add
    ///
	MSDK_EXPORT msdk_s8                                Store_CreditItem(msdk_StoreItem* p_item);

    ///
    /// Consume an item for the user.
    /// The amount field must be specified in order to remove the right amount to the user
    /// @param  p_item The item to consume
    ///
	MSDK_EXPORT msdk_s8                                Store_ConsumeItem(msdk_StoreItem* p_item);

    ///
    /// Restore the primary items of the user (used for a game reinstallation for instance)
    ///
	MSDK_EXPORT void                                Store_CallRestoreItems();

    ///
    /// Get the status of the CallRestoreItems request
    /// @return The status of the request
    ///
	MSDK_EXPORT msdk_Status                         Store_StatusRestoreItems();

    ///
    /// Get the result of the CallRestoreItems request
    /// @return The result of the request
    ///
	MSDK_EXPORT msdk_Result                         Store_ResultRestoreItems();

    ///
    /// Release the CallRestoreItems request
    ///
	MSDK_EXPORT void                                Store_ReleaseRestoreItems();

    
    ///
    /// Restore the durable secondary items (iOS only)
    /// You need to activate iCloud KeyValues to make it work
    /// Items are stored to iCloud so they are linked to iCloud account
    /// @return The list of the restored items (itemId is returned)
    ///
    MSDK_EXPORT Array_msdk_u32*                     Store_RestoreSecondaryItems();
    
    ///
    /// Release the memory allocated by Store_RestoreSecondaryItems function
    ///
    MSDK_EXPORT void                                Store_ReleaseRestoreSecondaryItems();
    
    ///
    /// Launch a request to buy a primary item
    /// @param  p_item The item to buy
    ///
	MSDK_EXPORT void                                Store_CallBuyPrimaryItem(msdk_PrimaryStoreItem* p_item);

    ///
    /// Get the status of a buy request
    /// @param  p_item The item in the buying process
    /// @return The status of the request
    ///
	MSDK_EXPORT msdk_Status                         Store_StatusBuyPrimaryItem(msdk_PrimaryStoreItem* p_item);

    ///
    /// Get the result of a buy request
    /// @param  p_item The item in the buying process
    /// @return The result of the request
    ///
	MSDK_EXPORT msdk_PrimaryPurchaseResult          Store_ResultBuyPrimaryItem(msdk_PrimaryStoreItem* p_item);

    ///
    /// Release a buy request
    /// @param  p_item The item in the buying process
    ///
	MSDK_EXPORT void                                Store_ReleaseBuyPrimaryItem(msdk_PrimaryStoreItem* p_item);

    ///
    /// Buy an item in the secondary store
    /// @param  p_item The item to buy
    /// @return The number of bought items now owned by the user
    ///
	MSDK_EXPORT Array_msdk_StoreItem*               Store_BuySecondaryItem(msdk_SecondaryStoreItem* p_item);

    ///
    /// Execute a custom request on the primary store
    /// @param  p_requestName The name of the request
    /// @return The result of the request
    ///
	MSDK_EXPORT Array_msdk_PrimaryStoreItem*        Store_PrimaryStoreRequest(const char_utf8* p_requestName);

    ///
    /// Execute a custom request on the secondary store
    /// @param  p_requestName The name of the request
    /// @return The result of the request
    ///
	MSDK_EXPORT Array_msdk_SecondaryStoreItem*      Store_SecondaryStoreRequest(const char_utf8* p_requestName);

    ///
    /// Get the transactions that weren't completed the first time but at launch
    /// Useful for server to server receipt validation
    /// @return The transactions done at resume
    ///
	MSDK_EXPORT Array_msdk_IAPProduct*        Store_GetResumedTransactions();

    ///
    /// Erase a transaction from the historic of the resumed transactions.
    /// @param p_ubiTransactionId The transaction id
    ///
	MSDK_EXPORT void                                Store_ClearResumedTransaction(const char_utf8* p_ubiTransactionId);

    ///
    /// Release memory allocated by Store_GetResumedTransactions
    ///
	MSDK_EXPORT void                                Store_ReleaseResumedTransactions();

    ///
    /// Register a callback that will be called when items have benn refreshed from the store (when the app goes back to foreground)
    /// @param p_calback The callback to register
    ///
    MSDK_EXPORT void                                Store_RegisterRefreshCallback(Store_RefreshCallback p_callback);
    
    MSDK_EXPORT void                                Store_RegisterRefreshCallbackUnity(long long p_callback);
#ifdef __cplusplus
}
#endif

#endif
