//
//  IAPManager.h
//  InAppPurchaseManagerTest
//
//  Created by Andrei Andreiana, Emanuel Jarnea on 9/18/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#ifndef InAppPurchaseManagerTest_IAPManager_h
#define InAppPurchaseManagerTest_IAPManager_h

#include "MobileSDK/common/Utils.h"
#include "MobileSDK/iab/IAPProduct.h"

///
/// These are possible result states for a transaction (purchase request)
///
enum msdk_TransactionStatus
{
    MSDK_TS_COMPLETED,  //!< Transaction was successful
    MSDK_TS_ALREADY_ENTITLED, //!< The pack is already owned
    MSDK_TS_FAILED, //!< The transaction failed with the 1st party
    MSDK_TS_CANCELED, //!< The user cancelled the transaction
    MSDK_TS_TIMEDOUT, //!< Unable to process the transaction due to bad connection
    MSDK_TS_REFUNDED, //!< The pack was refunded
    MSDK_TS_UNKNOWN_STATUS, //!< An unkown error occured
    MSDK_TS_INVALID_PRODUCT_ID, //!< The productId of the pack is not valid
    MSDK_TS_RECEIPT_VALIDATION_FAILED, //!< The receipt validation failed
    MSDK_TS_RECEIPT_VALIDATION_NETWORK_ERROR, //!< There was a network issue during receipt validation (The purchase with the 1st party has been made)
    MSDK_TS_LOADING, //!< Trying to buy while the store is initializing
    MSDK_TS_DEFERRED, //!< The transaction is deferred (will need approval of another user)
};

///
/// This is a result for a purchase request (buy product).
/// It contains information about the purchased product
/// and the purchase state.
///
typedef struct msdk_PurchaseResult
{
    msdk_TransactionStatus	transactionStatus;
    const msdk_IAPProduct*	product;
    const char_utf8*        receipt;
} msdk_PurchaseResult;

typedef struct msdk_ReceiptResult
{
    msdk_TransactionStatus  transactionStatus;
    const char_utf8*        ubiTransactionId;
}msdk_ReceiptResult;

#include "MobileSDK/common/Array.inl"
DECLARE_ARRAYS(msdk_PurchaseResult);


extern "C"
{
// Uninitializes all stores and frees memory
MSDK_EXPORT void IAPManager_Destroy();

MSDK_EXPORT msdk_s8 IAPManager_CanDestroy();

/// Initializes the in app billing module
MSDK_EXPORT void IAPManager_CallInit(Array_msdk_IAPProduct* p_products);

/// Checks the status of the initialization
/// @return The status of the initialization
MSDK_EXPORT msdk_Status IAPManager_StatusInit();

/// After IAPManager_StatusInit returned MSDK_ENDED, you should call IAPManager_ResultInit to get the know if the initialization was successfull or not.
/// @return MSDK_SUCCESS if successful, another value otherwise (@see msdk_Result for more info)
MSDK_EXPORT msdk_Result IAPManager_ResultInit();

/// Release the request for initialization
MSDK_EXPORT void IAPManager_ReleaseInit();

///
/// Used to query if In-App-Purchase is available to the client (user)
/// @return	MSDK_TRUE on success, MSDK_FALSE otherwise
///
MSDK_EXPORT char IAPManager_IsAvailable();

///
/// Used to query if In-App-Purchase has been initialized)
/// @return	MSDK_TRUE on success, MSDK_FALSE otherwise
///
MSDK_EXPORT char IAPManager_IsInit();

///
/// Get the product map of available and owned In-App-Purchase products.
/// @return		An array containing all the product ids and informations
///
MSDK_EXPORT const Array_msdk_IAPProduct* IAPManager_GetKnownProductMap();

///
/// Launch the purchase of a product based on the product ID.
/// The vendor's purchase pop-up will be presented to the user by the vendor purchase API.
/// @param		productId Product id string, for example "com.ubisoft.mygame.coin_pack1"
/// @return		The id of the request
///
MSDK_EXPORT msdk_s8 IAPManager_CallBuyUsingProductId(const char_utf8* productId);

///
/// Get the status of a purchase request.
/// @param		requestId	Id of the purchase request
/// @return		MSDK_ENDED if the request is finished, otherwise another value (@see msdk_Status for more information)
///
MSDK_EXPORT msdk_Status	IAPManager_StatusBuyUsingProductId(msdk_s8 requestId);

///
/// Get the result of the purchase request. Use this function after getting MSDK_ENDED while calling: IAPManager_StatusBuyUsingProductId.
/// @param		requestId	Id of the purchase request
/// @return		Returns a pointer to the msdk_PurchaseResult
///
MSDK_EXPORT msdk_PurchaseResult* IAPManager_ResultBuyUsingProductId(msdk_s8 requestId);

///
/// After a request has been completed (MSDK_ENDED) or timed-out (TIMEDOUT)
/// you must release the request.
/// Please copy your data you need from the result or use other functions
/// of the IAPManager, instead of keeping the request alive.
///
/// @param		requestId	Id of the purchase request
///
MSDK_EXPORT void IAPManager_ReleaseBuyUsingProductId(msdk_s8 requestId);


    ///
    /// Start the purchase of a pack with no internal receipt validation.
    /// This function only performs transaction with the store. It does not perform the receipt validation
    /// If you use this function you should then either use IAPManager_CallValidateReceipt or your own receipt valdiation system
    /// CallBuyPack requests cannot be done in paralel. You need to release it before processing a new one
    /// @param p_packId The id of the pack to initiate the purchase
    ///
    MSDK_EXPORT void IAPManager_CallBuyPack(const char_utf8* p_packId);

    ///
    /// Returns the status of the CallBuyPack request
    /// @return The status of the request
    ///
    MSDK_EXPORT msdk_Status IAPManager_StatusBuyPack();

    ///
    /// Returns the result of the CallBuyPack request
    /// @return The result of the request
    ///
    MSDK_EXPORT msdk_PurchaseResult* IAPManager_ResultBuyPack();

    ///
    /// Release the CallBuyPack request
    /// Have to be called to allow to perform other CallBuyPack requests
    ///
    MSDK_EXPORT void IAPManager_ReleaseBuyPack();

    ///
    /// Restore managed packs bought by the user
    /// <strong>CAUTION : This request does not perform receipt validation.</strong>
    ///
    MSDK_EXPORT void IAPManager_CallRestorePacks();

    ///
    /// Return the status of the CallRestorePacks request
    /// @return The status of the request
    ///
    MSDK_EXPORT msdk_Status IAPManager_StatusRestorePacks();

    ///
    /// Return the result of the CallRestorePacks request
    /// @return The result of the request
    ///
    MSDK_EXPORT Array_msdk_PurchaseResult* IAPManager_ResultRestorePacks();

    ///
    /// Release the CallRestorePacks request
    ///
    MSDK_EXPORT void IAPManager_ReleaseRestorePacks();

    ///
    /// Start to validate a receipt with Houston servers
    /// Several receipt validation can be done in parallel
    /// @param p_receipt The receipt to valdiate
    /// @param p_productIdentifier The id of the pack to validate
    /// @param p_formattedPrice The formatted price of the pack
    /// @return The id of the request
    ///
    MSDK_EXPORT msdk_s8 IAPManager_CallValidateReceipt(const char_utf8* p_receipt, const char_utf8* p_productIdentifier, const char_utf8* p_formattedPrice);

    ///
    /// Returns the status of a validation request
    /// @param p_requestId The id of the request
    /// @return The status of the request
    ///
    MSDK_EXPORT msdk_Status IAPManager_StatusValidateReceipt(msdk_s8 p_requestId);

    ///
    /// Returns the result of a validation request
    /// @param p_requestId The id of the request
    /// @return The status of the transaction
    ///
    MSDK_EXPORT msdk_ReceiptResult* IAPManager_ResultValidateReceipt(msdk_s8 p_requestId);

    ///
    /// Release a validation request
    /// @param p_requestId The id of the request
    ///
    MSDK_EXPORT void IAPManager_ReleaseValidateReceipt(msdk_s8 p_requestId);

MSDK_EXPORT void                    IAPManager_CallValidateItem(const char_utf8* p_productId);

MSDK_EXPORT msdk_Status				IAPManager_StatusValidateItem(const char_utf8* p_productId);

MSDK_EXPORT msdk_PurchaseResult* 	IAPManager_ResultValidateItem(const char_utf8* p_productId);

MSDK_EXPORT void 					IAPManager_ReleaseValidateItem(const char_utf8* p_productId);

///
/// Use this function to create an request to consume a product.
/// This is available only for some vendors and some product types
/// @param		productId	Product id string.
/// @return		The id of the load store request
///
MSDK_EXPORT msdk_s8 IAPManager_CallConsumeProduct(const char_utf8* productId);

///
/// Get the result of the consumption request.
/// Use this function after you have checked for an MSDK_ENDED request state.
/// @param		requestId	Id of the consumption request
/// @return		Returns a pointer to the PurchaseRequestResult
///
MSDK_EXPORT msdk_PurchaseResult* IAPManager_ResultConsumeProduct(msdk_s8 requestId);

///
/// Get the status of a consumption request.
/// @param		requestId	Id of the consumption request
/// @return		Returs the request state.
///
MSDK_EXPORT msdk_Status	IAPManager_StatusConsumeProduct(msdk_s8 requestId);

///
/// After a request has been completed (MSDK_ENDED) or timed-out (TIMEDOUT)
/// you must release the request.
/// Please copy your data you need from the result or use other functions
/// of the IAPManager, instead of keeping the request alive.
/// @param		requestId	Id of the consumption request
///
MSDK_EXPORT void IAPManager_ReleaseConsumeProduct(msdk_s8 requestId);

///
/// Restore the purchase of a user. Once the request if finished you can call
/// to IAPManager_GetKnownProductMap and check the status of product to see which ones are owned.
/// On Google play and Amazon this call is made silently during the initialization of the IAB.
/// On iOS the call is <b>not</b> made during the initialization, because it <b>display a login window from apple</b>.
/// So you should call it only when the user press the "Restore purchase" button on iOS.
///
MSDK_EXPORT void IAPManager_CallRestorePurchase();

///
/// Status of the restore purchase request
/// @return MSDK_ENDED if the request is finished.
///
MSDK_EXPORT msdk_Status	IAPManager_StatusRestorePurchase();

///
/// Result of the restore purchase request.
/// Must be called after receiving MSDK_ENDED from IAPManager_StatusRestorePurchase.
/// @return MSDK_SUCCESS if the request is successfull.
///
MSDK_EXPORT msdk_Result IAPManager_ResultRestorePurchase();

///
/// Release the restore purchase request.
/// Must be called after you finish checking the result of the function: IAPManager_ResultRestorePurchase
///
MSDK_EXPORT void IAPManager_ReleaseRestorePurchase();

    ///
    /// Get the historic of the transactions performed. Only productId and ubiTransactionId will be available
    /// @return A list of IAPProduct
    ///
MSDK_EXPORT Array_msdk_IAPProduct*  IAPManager_GetTransactionHistoric();

    ///
    /// Erase the entry of a transaction in the historic
    /// @param p_ubiTransactionId The ubiTransactionId of the transaction to erase
    ///
MSDK_EXPORT    void    IAPManager_ClearTransactionFromHistoric(const char_utf8* p_ubiTransactionId);

    ///
    /// Clear the historic of transactions
    ///
MSDK_EXPORT    void IAPManager_ClearTransactionHistoric();

///
/// !! Samsung Apps specific function !!
/// When the result of IAPManager_ResultInit is MSDK_SAMSUNG_IAP_NEED_UPDATE or MSDK_SAMSUNG_IAP_NOT_INSTALLED
/// on samsung platform, you need to display a message to the user and invite him to install or update the in app purchase application.
/// By calling that function you'll open the samsung app store at the right application page to install / update samsung in app purchase application.
/// This is a limitation imposed by samsung, not the msdk.
///
MSDK_EXPORT void IAPManager_SamsungApps_IapRedirection();

    ///
    /// Returns the country code of the store the user is using
    /// Must be called after the init of the IAPManager
    /// @return The store country code on success. NULL otherwise(at least 1 valid item is required)
    ///
MSDK_EXPORT const char_utf8* IAPManager_StoreCountryCode();

}
#endif
