#ifndef MOBILE_SDK_API_ACHIEVEMENT_H
#define MOBILE_SDK_API_ACHIEVEMENT_H

#include "MobileSDK/common/Utils.h"
#include "MobileSDK/social/UserAchievement.h"

#ifdef __cplusplus
extern "C"
{
#endif

///
/// Launch a request to update/add the state of an achievement
/// |    iOS           |      Android       |
/// | :--------------: | :----------------: |
/// | MSDK_GAME_CENTER |                    |
/// | MSDK_FACEBOOK    | MSDK_FACEBOOK      |
/// |                  | MSDK_GAME_CIRCLE   |
/// |                  | MSDK_GAME_SERVICES |
/// @param  p_achievement an Achievement descriptor, you should get it from Achievement_CallGameAchievements.
///     If you want/need to build it manually, this function require:
///         p_achievement->achievementId
///         p_achievement->networkId
///         p_achievement->percentComplete
///         p_achievement->achievementUrl
/// @return The id of the request, -1 if an error occurs
///
MSDK_EXPORT msdk_s8  Achievement_CallUpdateAchievement(msdk_UserAchievement* p_achievement);

///
/// Get the status of an achievement update request
/// @param  p_requestId The id of the request
/// @return The status of the request
///
MSDK_EXPORT msdk_Status  Achievement_StatusUpdateAchievement(msdk_s8 p_requestId);

///
/// Get the result of an achievement's update
/// @param  p_requestId the id of the request
/// @return The result of the request
///
MSDK_EXPORT msdk_Result Achievement_ResultUpdateAchievement(msdk_s8 p_requestId);

///
/// Release an update request
/// @param  p_requestId The id of the request
///
MSDK_EXPORT void Achievement_ReleaseUpdateAchievement(msdk_s8 p_requestId);

///
/// Launch a request to retrieve the achievements of a user
/// |      iOS         |      Android       |
/// | :--------------: | :----------------: |
/// | MSDK_GAME_CENTER |                    |
/// | MSDK_FACEBOOK    | MSDK_FACEBOOK      |
/// |                  | MSDK_GAME_CIRCLE   |
/// |                  | MSDK_GAME_SERVICES |
/// @param  p_networkId	the id of the network where the achievements are stored
///
MSDK_EXPORT void  Achievement_CallUserAchievements(msdk_NetworkId p_network);

///
/// Launch a request to retrieve the achievements of a game
/// |    iOS           |      Android       |
/// | :--------------: | :----------------: |
/// | MSDK_GAME_CENTER |                    |
/// | MSDK_FACEBOOK    | MSDK_FACEBOOK      |
/// |                  | MSDK_GAME_CIRCLE   |
/// |                  | MSDK_GAME_SERVICES |
/// @param  p_networkId the id of the network where the achievements are stored
///
MSDK_EXPORT void  Achievement_CallGameAchievements(msdk_NetworkId p_network);

///
/// Launch a request to retrieve the achievements of friends of a user
/// Launch a request to retrieve the achievements of a game
/// |      iOS         | Android       |
/// | :--------------: | :-----------: |
/// | MSDK_GAME_CENTER |               |
/// | MSDK_FACEBOOK    | MSDK_FACEBOOK |
/// @param  p_networkId the id of the network where the achievements are stored
/// @return The id of the request on success, -1 otherwise
///
MSDK_EXPORT void  Achievement_CallFriendsAchievements(msdk_NetworkId p_network, Array_msdk_UserInfo* p_userInfos);

///
/// Get the status of an achievements retrieving request
/// @param  p_requestId The id of the request
/// @return The status of the request
///
MSDK_EXPORT msdk_Status   Achievement_StatusAchievements(msdk_NetworkId p_network);

///
/// Get the achievements retrieved in by a request
/// @param p_requestId The id of the request
/// @return The achievements retrived by the request
///
MSDK_EXPORT Array_msdk_UserAchievement* Achievement_ResultAchievements(msdk_NetworkId p_network);

///
/// Release a retrieving request
/// @param  p_requestId The id of the request
///
MSDK_EXPORT void Achievement_ReleaseAchievements(msdk_NetworkId p_network);

///
/// Display the achievement interface of the platform (works for GameCenter, GameCircle and GameServices)
/// @param  p_networkId The id of the network (MSDK_GAME_CENTER, MSDK_GAME_CIRCLE, MSDK_GAME_SERVICES)
///
MSDK_EXPORT void Achievement_ShowNativeInterface(msdk_NetworkId p_networkId);
MSDK_EXPORT void Achievement_CallNativeInterface(msdk_NetworkId p_networkId);
MSDK_EXPORT msdk_Status Achievement_StatusNativeInterface(msdk_NetworkId p_networkId);
MSDK_EXPORT msdk_Result Achievement_ResultNativeInterface(msdk_NetworkId p_networkId);
MSDK_EXPORT void Achievement_ReleaseNativeInterface(msdk_NetworkId p_networkId);

///
/// Activate or unactivate the platform's completion banner when an achievement is unlocked (works only for GameCenter)
/// @param  p_isShown MSDK_TRUE if you want to activate it, MSDK_FALSE otherwise
///
MSDK_EXPORT void Achievement_ShowCompletionBanner(char p_isShown);

///
/// Reset all the achievements owned by the user on a specific network
/// @param p_networkId The id of the network
///
MSDK_EXPORT void Achievement_ResetAchievements(msdk_NetworkId p_networkId);

#ifdef __cplusplus
}
#endif

#endif // MOBILE_SDK_API_ACHIEVEMENT_H
