#ifndef MOBILE_SDK_API_LEADERBOARD_H
#define MOBILE_SDK_API_LEADERBOARD_H

#include "MobileSDK/common/Utils.h"
#include "MobileSDK/social/Score.h"

typedef enum msdk_TimeScope    {MSDK_TODAY = 0x1, MSDK_WEEK, MSDK_MONTH, MSDK_YEAR, MSDK_ALLTIME}msdk_TimeScope;

typedef enum msdk_RequestScope {MSDK_GLOBAL = 0x1, MSDK_FRIENDS, MSDK_NOT_A_SCOPE }msdk_RequestScope;

typedef struct msdk_LeaderboardOptions
{
    const char_utf8*    group;
    const char_utf8*    country;
    msdk_RequestScope   requestScope;
    msdk_TimeScope      timeScope;
}msdk_LeaderboardOptions;

enum msdk_LeaderboardStatType
{
    MSDK_ST_GLOBAL,
    MSDK_ST_COUNTRY,
    MSDK_ST_GROUP
};

#ifdef __cplusplus
extern "C"
{
#endif
///
/// Send a request to add a score on a/several leaderboard(s)
/// |     iOS          |      Android       |
/// | :--------------: | :----------------: |
/// | MSDK_GAME_CENTER |                    |
/// | MSDK_FACEBOOK    | MSDK_FACEBOOK      |
/// |                  | MSDK_GAME_CIRCLE   |
/// |                  | MSDK_GAME_SERVICES |
/// @param  p_mask A bit mask that represents the leaderboards to add a score to
///         If p_mask is 0 then all leaderboards are filled
/// @param  p_leaderBoardId The id or name of the leaderboard to add the score
/// @param  p_value The score to add to the leaderboard
/// @return The id of the created request as a s8, -1 if an error occured
///
MSDK_EXPORT msdk_s8 	Leaderboard_CallPublishScore	(msdk_NetworkId p_network, const char_utf8* p_leaderBoardId, const char_utf8* p_value);
    
///
/// Get the state of a request
/// @param  p_requestID The id of the request
/// @return The state of the request
///
MSDK_EXPORT msdk_Status	Leaderboard_StatusPublishScore	(msdk_s8 p_requestId);

///
/// Get the result of an AddScore request
/// @param p_requestID The id of the request
/// @return The result of the request as a E_result
/// |     Value	                 |    Description   |
/// | :------------------------: | :--------------: |
/// | MSDK_SUCCESS               | The score has been successfully published |
/// | MSDK_NETWORK_ERROR         | An error occured while publishing the score. Check the internet connection before making the call. |
/// | MSDK_NETWORK_NOT_CONNECTED | Trying to published a score on a network where the user is not connected |
/// | MSDK_BAD_PARAMETER         | The score send as parameter can't be parsed as a long |
///
MSDK_EXPORT msdk_Result Leaderboard_ResultPublishScore	(msdk_s8 p_requestId);

///
/// Release a request created to add a score
/// @param  p_requestID The id of the request
///
MSDK_EXPORT void		Leaderboard_ReleasePublishScore	(msdk_s8 p_requestId);

///
/// Send a request to get the scores of the player's friends or all the scores
/// |      iOS           | Android       |
/// | :----------------: | :-----------: |
/// | MSDK_GAME_CENTER   |               |
/// | MSDK_FACEBOOK      | MSDK_FACEBOOK |
/// @param  p_mask  A bit mask that represents the leaderboards to add a score to
///         If p_mask is 0 then all leaderboards are filled
/// @param  p_scope The scope of the request (MSDK_FRIENDS for retrieve only friends' scores or MSDK_GLOBAL to retrieve all the scores). User's score is always part of the result
///                 MSDK_GLOBAL is not supported on MSDK_FACEBOOK
/// @param  p_leaderBoardId The id or name of the leaderboard to add the score
/// @param  p_location  MSDK_GAME_CENTER & MSDK_GAME_CIRCLE only: The index of the first score to get in the list (1 is the best score)
/// @param  p_length    MSDK_GAME_CENTER & MSDK_GAME_CIRCLE only: The number of scores to retrieve
/// @param  p_timescope MSDK_GAME_CENTER & MSDK_GAME_CIRCLE only: The period from which the scores should be taken from
/// @return The id of the created request as an s8
///
MSDK_EXPORT void 	Leaderboard_CallScoresList(msdk_NetworkId p_network, msdk_RequestScope p_scope, const char_utf8* p_leaderBoardId, msdk_u32 p_location, msdk_u32 p_length, const msdk_TimeScope p_timescope);

///
/// Send a request to get the scores of the player's friends or all the scores surrounding a player within a range (the player's score will be in the middle of scoresNumber if possible)
/// |      iOS           |      Android       |
/// | :----------------: | :----------------: |
/// | MSDK_GAME_CENTER   |                    |
/// | MSDK_FACEBOOK      | MSDK_FACEBOOK      |
/// |                    | MSDK_GAME_SERVICES |
/// @param  p_mask  A bit mask that represents the leaderboards to add a score to
///         If p_mask is 0 then all leaderboards are filled
/// @param  p_scope The scope of the frequest (MSDK_FRIENDS for retrieve only friends' scores or MSDK_GLOBAL to retrieve all the scores). User's score is always part of the result
///                 MSDK_GLOBAL is not supported on MSDK_FACEBOOK
/// @param  p_leaderBoardId The id or name of the leaderboard to add the score
/// @param  p_scoresNumber  The number of scores to retrieve (local player's score will be in the mifddle of these number of scores if possible)
/// @param  MSDK_GAME_CENTER only: p_timescope The period from which the scores should be taken from
/// @return The id of the created request as an s8
///
MSDK_EXPORT void 	Leaderboard_CallSurroundingScoresList(msdk_NetworkId p_network, msdk_RequestScope p_scope, const char_utf8* p_leaderBoardId, msdk_u32 p_scoresNumber, const msdk_TimeScope p_timescope);

///
/// Send a request to get the scores of the given players
/// |         iOS        | Android       |
/// | :----------------: | :-----------: |
/// | MSDK_GAME_CENTER   |               |
/// | MSDK_FACEBOOK      |               |
/// @param  p_mask  A bit mask that represents the leaderboards to add a score to
///         If p_mask is 0 then all leaderboards are filled
/// @param  p_leaderBoardId The id or name of the leaderboard to add the score
/// @param  p_timeScope The period from which the scores should be taken from
/// @return The id of the created request as an s8
///
MSDK_EXPORT void 	Leaderboard_CallPlayersScoresList(msdk_NetworkId p_network, const char_utf8* p_leaderBoardId, PointerArray_msdk_UserInfo*, const msdk_TimeScope p_timeScope);

///
/// Get the state of a request
/// @param  p_requestID The id of the request
/// @return The state of the request
///
MSDK_EXPORT msdk_Status Leaderboard_StatusScoresList(msdk_NetworkId p_network);

///
/// Get the scores retrieved by a request
/// @param  p_requestID The id of the request that retrieved the scores
/// @return A list of UserScore* that represents the scores retrieved
///
MSDK_EXPORT Array_msdk_Score* 	Leaderboard_ResultScoresList(msdk_NetworkId p_network);

///
/// Release a request created to retrieve scores
/// @param  p_requestID The id of the request
///
MSDK_EXPORT void 		Leaderboard_ReleaseScoresList(msdk_NetworkId p_network);

///
/// Display the leaderboard interface of the platform
/// |         iOS        |      Android       |
/// | :----------------: | :----------------: |
/// | MSDK_GAME_CENTER   |                    |
/// |                    | MSDK_GAME_CENTER   |
/// |                    | MSDK_GAME_SERVICES |
/// @param  p_networkId The id of the network (MSDK_GAME_CENTER, MSDK_GAME_CIRCLE)
/// @param p_leaderboardId The id of the leaderboard you want to be shown (can be NULL)
/// @return The result
/// |     Value	       |    Description   |
/// | :--------------: | :--------------: |
/// | MSDK_SUCCESS     | The interface has been shown |
/// | MSDK_NETWORK_NOT_CONNECTED | Trying to show the interface on a network where the user is not connected |
///
MSDK_EXPORT msdk_Result		Leaderboard_ShowNativeInterface(msdk_NetworkId p_networkId, const char_utf8* p_leaderbaordId);

    
    ///
    /// Create a score to be registered later
    /// @param p_service The service to register the score
    /// @param p_leaderboardId The id of the leaderboard
    /// @param p_value The value of the score
    /// @return The created score
    ///
MSDK_EXPORT    msdk_Score*     Leaderboard_CreateScore(msdk_Service p_service, const char_utf8* p_leaderBoardId, const char_utf8* p_value);
    
    ///
    /// Houston leaderboards only. Add the user name to the score
    /// @param p_score The score to configure
    /// @param p_name The name of the player
    ///
MSDK_EXPORT    void Leaderboard_AddName(msdk_Score* p_score, const char_utf8* p_name);
    
    ///
    /// Houston leaderboards only. Set a group for a score
    /// @param p_score The score to configure
    /// @param p_group The group to add
    ///
MSDK_EXPORT    void Leaderboard_AddGroup(msdk_Score* p_score, const char_utf8* p_group);
    
    ///
    /// Start a request to register a score
    /// @param p_score The score to register
    /// @return The id of the request
    ///
MSDK_EXPORT    msdk_s8 	Leaderboard_CallRegisterScore(msdk_Score* p_score);
    
    ///
    /// Get the status of a RegisterScore request
    /// @param p_requestId The id of the request
    /// @return The status of the request
    ///
MSDK_EXPORT    msdk_Status Leaderboard_StatusRegisterScore(msdk_s8 p_requestId);

    ///
    /// Get the result of a RegisterScore request
    /// @param p_requestId The id of the request
    /// @return The result of the request
    ///
MSDK_EXPORT    msdk_Result Leaderboard_ResultRegisterScore(msdk_s8 p_requestId);
    
    ///
    /// Release a RegisterScore request
    /// @param p_requestId The id of the request
    ///
MSDK_EXPORT    void Leaderboard_ReleaseRegisterScore(msdk_s8 p_requestId);
    
    ///
    /// Start a request to get a leaderboard
    /// @param p_service The service to request
    /// @param p_leaderboardid The leaderboard to fetch
    /// @param p_location The starting index in the leaderboard(1 is top)
    /// @param p_length The length of the leaderboard
    /// @param p_options Additional options of the request
    ///
MSDK_EXPORT    void 	Leaderboard_CallScores(msdk_Service p_service, const char_utf8* p_leaderBoardId, msdk_u32 p_location, msdk_u32 p_length, msdk_LeaderboardOptions p_options);
    
    ///
    /// Start a request to get a leaderboard with scores surrounding the player
    /// @param p_service The service to request
    /// @param p_leaderboardid The leaderboard to fetch
    /// @param p_length The number of scores to retrieve
    /// @param p_options Additional options of the request
    ///
MSDK_EXPORT    void 	Leaderboard_CallSurroundingScores(msdk_Service p_service, const char_utf8* p_leaderBoardId, msdk_u32 p_scoresNumber, msdk_LeaderboardOptions p_options);
    
    ///
    /// Start a request to get a leaderboard with specific users
    /// @param p_service The service to request
    /// @param p_leaderboardid The leaderboard to fetch
    /// @param p_users The players to use
    /// @param p_options Additional options of the request
    ///
MSDK_EXPORT    void 	Leaderboard_CallPlayersScores(msdk_Service p_service, const char_utf8* p_leaderBoardId, PointerArray_msdk_UserInfo* p_users, msdk_LeaderboardOptions p_options);
    
    ///
    /// Get the status of the current CallScores request
    /// @param p_service The service to use
    /// @return The status of the request
    ///
MSDK_EXPORT    msdk_Status Leaderboard_StatusScores(msdk_Service p_service);
    
    ///
    /// Get the result of the current CallScores request
    /// @param p_service The service used
    /// @return The retrieved scores
    ///
MSDK_EXPORT    Array_msdk_Score* Leaderboard_ResultScores(msdk_Service p_service);
    
    ///
    /// Release the current CallScores request
    /// @param p_service The service used
    ///
MSDK_EXPORT    void Leaderboard_ReleaseScores(msdk_Service p_service);
    
    /// Houston only. Fetch the stats of a leaderboard
    /// @param p_leaderboardId The id of the leaderboard
    /// @param p_type The type of stats you want to get
    ///
MSDK_EXPORT    void Leaderboard_CallStats(const char_utf8* p_leaderboardId, msdk_LeaderboardStatType p_type);
    
    ///
    /// Get the status of a stats request
    /// @return Get the status of the request
    ///
MSDK_EXPORT    msdk_Status Leaderboard_StatusStats();
    
    ///
    /// Get the result of a stats request
    /// @return The stats of the leaderboard
    ///
MSDK_EXPORT    const char_utf8* Leaderboard_ResultStats();
    
    ///
    /// Release a stats request
    ///
MSDK_EXPORT    void Leaderboard_ReleaseStats();

#ifdef __cplusplus
}
#endif

#endif // MOBILE_SDK_API_LEADERBOARD_H
