//
// BaseMessage.h
//
// Created on: 30 oct. 2012
//

#ifndef MESSAGING_H
#define MESSAGING_H

#include "MobileSDK/common/Utils.h"
#include "MobileSDK/social/UserInfo.h"

typedef enum msdk_FacebookMode
{
	MSDK_NO_DIALOG = 0,
	MSDK_DEFAULT_DIALOG = 1, //with the fallback in webview if app not installed
	MSDK_ONLY_FEED_DIALOG = 2 // deprecated
} msdk_FacebookMode;

typedef enum msdk_ShareType
{
    MSDK_FACEBOOK_LINK = 0,
    MSDK_FACEBOOK_PHOTO = 1,
    MSDK_FACEBOOK_VIDEO = 2,
    MSDK_FACEBOOK_OG_STORY = 3
} msdk_ShareType;

#define msdk_BaseMessage msdk_Message

///
/// Structure to send or post messages to a user
///
typedef struct msdk_Message
{
    const msdk_UserInfo* 	recipient;
    char_utf8* 		message;
    char_utf8* 		title;
    msdk_ShareType type;
    
	// #ifdef MSDK_BUILD_LIB
	//     msdk_extras* 		extras;
	// #else
    // 	void* internalBatching;
	// #endif
	void* 		extras;
}msdk_Message;

#ifdef __cplusplus
extern "C"
{
#endif
    

    ///
    /// Initialize a message to send through WallPost
    /// @param p_message The content of the message. Could be NULL
    /// @param p_userInfo The user to send the message to. Could be NULL
    /// @return The created message
    ///
    MSDK_EXPORT msdk_Message* Message_Create(const char_utf8* p_title, const char_utf8* p_message, const msdk_UserInfo* p_userInfo);

    ///
    /// Initialize a message to send through WallPost
    /// @param p_message The message struct to fill.
    /// @param p_type The type to specify
    ///
    MSDK_EXPORT void Message_AddShareType(msdk_Message* p_message, msdk_ShareType p_type);

	///
	/// Specialize a link for Facebook, your need to call <strong>::Message_AddLink</strong> before calling this function.
	/// @param p_message The message to configure
	/// @param p_linkName The name of the link
	/// @param p_linkCaption The caption of the message
	/// @pram p_linkDescription The description of the link
	///
	MSDK_EXPORT void Message_AddLinkFacebook(msdk_Message* p_message, const char_utf8* p_linkName, const char_utf8* p_linkCaption, const char_utf8* p_linkDescription);

	///
	/// Add a link to a message
	/// @param p_message The message to configure
	/// @param p_link The link
	///
	MSDK_EXPORT void Message_AddLink(msdk_Message* p_message, const char_utf8* p_link);

    ///
    /// Add a picture stored on a server to a message. <strong>You need to call ::Message_AddLink</strong> before calling this function.
    /// This function is compatible with ::Message_AddLinkFacebook
    /// @param p_message The message to configure
    /// @param p_pictureUrl The url of the picture
    ///
    MSDK_EXPORT void Message_AddPictureUrl(msdk_Message* p_message, const char_utf8* p_pictureUrl);

    ///
    /// Add a video stored on localy to a message.
    /// @param p_message The message to configure
    /// @param p_videoPath The path of the video
    ///
    MSDK_EXPORT void Message_AddVideo(msdk_Message* p_message, const char_utf8* p_videoPath);

    ///
    /// add a user tag to your message
    /// @param p_message The message to configure
    /// @param p_tag The id of the user to tag
    ///
    MSDK_EXPORT void Message_AddTagFacebook(msdk_BaseMessage* p_message, const char_utf8* p_tag);
    
    ///
    /// Add a icon stored on a server to a message.
    /// @param p_message The message to configure
    /// @param p_icon The url of the icon
    ///
    MSDK_EXPORT void Message_AddIconFacebook(msdk_BaseMessage* p_message, const char_utf8* p_icon);


	///
	/// Add a Facebook action/object to a message
	/// @param p_message The message to configure
	/// @param p_action The action (write, cook, ...)
	/// @param p_objectType  The type of the object (book, receipe, ...)
	/// @param p_object The URL of the object. Set to NULL if you want to publish a user generated object
	///
	MSDK_EXPORT void Message_AddActionObjectFacebook(msdk_Message* p_message, const char_utf8* p_action, const char_utf8* p_objectType, const char_utf8* p_objectUrl, msdk_s8 p_isExplicit);

    ///
    /// Add a property to the action of the ActionObject.
    /// Allows you to add extra datas to your stories.
    /// Example : John Doe has reach level1 in 10 minutes. Here then minutes is given by the game trough a property on the action
    /// @param p_message The message to configure
    /// @param p_key The property name
    /// @param p_value The property value
    ///
    MSDK_EXPORT void    Message_AddActionPropertyFacebook(msdk_Message* p_message, const char_utf8* p_key, const char_utf8* p_value);

    ///
    /// Add a property to the object of the ActionObject.
    /// Allows you to add extra datas to your stories.
    /// Example : John Doe has reach level1 named TheLevel in 10 minutes. Here then the name is given by the game trough a property on the object
    /// @param p_message The message to configure
    /// @param p_key The property name
    /// @param p_value The property value
    ///
    
    MSDK_EXPORT void    Message_AddObjectPropertyFacebook(msdk_Message* p_message, const char_utf8* p_key, const char_utf8* p_value);
    ///
    /// Customize the previously added object. It allows you to publish user generated stories
    /// @param p_message The message to configure
    /// @param p_title The title of the object
    /// @param p_imageUrl The url of the image of the object (can be NULL)
    /// @param p_url The url of the object(can be NULL)
    /// @param p_description The description of the object
    /// @param
    ///
    MSDK_EXPORT void    Message_CustomizeObjectfacebook(msdk_Message* p_message, const char_utf8* p_title, const char_utf8* p_imageUrl, const char_utf8* p_url, const char_utf8* p_description);


    ///
    /// Add a Facebook picture stored locally on the device, and will upload it.
    /// @param p_message The message to configure
	/// @param p_picturePath The path on the disk of the picture that will be posted
    ///
    MSDK_EXPORT void Message_AddPicture(msdk_Message* p_message, const char_utf8* p_picturePath);

	///
	/// Specify that a WallPost will use the facebook dialog (not the game UI). This way the user is able to modify the text, tag user, etc ...
	/// @param p_message The message to configure
	/// @param MSDK_DEFAULT_DIALOG if you want to use the facebook dialogue, MSDK_NO_DIALOG if you want to use your game UI to handle the WallPost
	///
	MSDK_EXPORT void Message_UseFacebookDialog(msdk_Message* p_message, msdk_FacebookMode p_mode);

	///
	/// Release the memory allocated for a message
	/// @param p_message The message to release
	///
	MSDK_EXPORT void Message_Release(msdk_Message* p_message);

#ifdef __cplusplus
}
#endif

#endif // MESSAGING_H
