#ifndef MOBILESDK_NOTIFICATION_H_
# define MOBILESDK_NOTIFICATION_H_

#include "MobileSDK/common/Utils.h"


#ifdef __cplusplus
extern "C"
{
#endif

    ///
    /// The parameters of the callback are:
    ///     - A string containing all the pushes received in json format.
    ///     - An int that identify the index of the push used to launch the game, or -1 if no push was used to launch the game.
    ///
    typedef void (*Notification_PushCallback)(const char_utf8* p_notifications, msdk_s32 p_indexPnLaunched);

    ///
    /// Register a game callback that will be called each time we receive a push notification.
    /// The behaviour is the following:
    ///     - If the game is running, the callback is called when the push is received.
    ///     - If the game is in background or is not runing at all the callback will be call at game launch.
    /// Limitations:
    ///     - On iOS, if the game is in background and we receive a push, it'll be present in the callback only if the user accept the push.
    ///     - On iOS, if the game is in background and we receive 3 pushes, if the user click on the push, only the latest push received will be present in the callback at launch.
    ///     - On Android, from Lollipop 5.0, notification images must be strictly white on transparent images. The background transparent color will be replaced by your color resource with the name "msdk_notification_color"
    /// @param the callback to be called when a push notification is received
    ///
    void Notification_RegisterPushCallback(Notification_PushCallback p_callback);

    ///
    /// Asks the user if he wants to receive notifications
    /// On iOS it'll display a popup to the user the first time it is called so he can choose either to accept or refuse to receive local or server notifications from your game.
    /// Useless on Android devices.
    ///
    void Notification_Register();

    ///
    /// Explicitly register the device for push notification.
    /// Must be called in the following callback :
    ///-(void) application:(UIApplication *)application didRegisterUserNotificationSettings:(UIUserNotificationSettings *)notificationSettings
    /// This callback will be called after a call to Notification_Register
    ///
    void Notification_RegisterForPush();

    ///
    /// Return is the game is ready to receive push notification.
    /// On iOS it'll return MSDK_TRUE only after the user accept the popup asking to receive push notification.
    /// On android it'll MSDK_TRUE all the time is GCM is available on the device (Not on kindle fire for example).
    /// @return MSDK_TRUE is the game is ready to receive push notification, MSDK_FALSE otherwise.
    ///
    msdk_s8 Notification_IsRegisteredForPush();

    enum msdk_CalendarUnit
    {
        MSDK_CU_SECOND,
        MSDK_CU_MINUTE,
        MSDK_CU_HOUR,
        MSDK_CU_DAY,
        MSDK_CU_WEEKDAY,
        MSDK_CU_WEEK,
        MSDK_CU_MONTH,
        MSDK_CU_YEAR,
        MSDK_CU_NONE,
    };

    typedef struct msdk_LocalNotification
    {
/*#ifdef MSDK_BUILD_LIB
		msdk_LocalNotificationInfos* infos;
#else
		void* internalData;
#endif
*/
        void* infos;
    }msdk_LocalNotification;

#include "MobileSDK/common/Array.inl"
DECLARE_ARRAYS(msdk_LocalNotification);

    ///
    /// IOS | ANDROID
    ///	Allocate but don't schedule a new local notification
    ///	@return A pointer to a new local notification
    ///
    msdk_LocalNotification*	Notification_CreateLocal();

    ///
    /// IOS | ANDROID
    ///	Release the memory taken by a LocalNotification, but do not unschedule it !
    ///	@param A pointer to a local notification to free
    ///
    void	Notification_ReleaseLocal(msdk_LocalNotification* p_ln);

    ///
    /// IOS | ANDROID
    ///	Set the date (in the future) when the local notification will be displayed
    ///	@param p_ln the pointer to the local notification to be configured
    /// @param p_date A date IN THE FUTURE when the local notification will be displayed. Format: yyyy-mm-dd hh:mm:ss
    ///
    void Notification_SetFireDate(msdk_LocalNotification* p_ln, const char_utf8* p_date);

    ///
    /// Set in how much time from now the local notification will be trigerred
    /// @param p_ln The local notification to setup
    /// @param p_seconds The number of seconds before the push will be displayed to the user
    ///
    void Notification_SetFireTime(msdk_LocalNotification* p_ln, msdk_u32 p_seconds);

    ///
    /// Configure a local notification to repeat itself
    /// @param p_ln The local notification to configure
    /// @param m_interval The repeat interval
    ///
    void Notification_SetRepeatInterval(msdk_LocalNotification* p_ln, msdk_CalendarUnit m_interval);

    ///
    /// IOS | ANDROID
    ///	Set the icon to be displayed when the local notification appear
    ///	@param p_pn the pointer to the local notification to be configured
    /// @param p_iconPath a path to a PNG file
    ///
    void Notification_SetIcon(msdk_LocalNotification* p_ln, const char_utf8* p_iconPath);

    ///
    /// IOS | ANDROID
    ///	Set the message body of the local notification
    ///	@param p_pn the pointer to the local notification to be configured
    /// @param p_message a string containing the body of the local notification
    ///
    void Notification_SetBody(msdk_LocalNotification* p_ln, const char_utf8* p_message);

    ///
    /// IOS | ANDROID
    ///	Set the sound to be play when the push is displayed
    ///	@param p_pn the pointer to the local notification to be configured
    /// @param p_soundName The name of a sound file. On Android the sound must be placed directly at the root of the assets folder and must be an mp3
    ///
    void Notification_SetSound(msdk_LocalNotification* p_ln, const char_utf8* p_soundName);


    ///
    /// IOS | ANDROID
    /// Specify if an alert will be displayed
    /// @param p_ln The notification to configure
    /// @param p_isDisplayed MSDK_TRUE to display the laert, MSDK_FALSE otherwise
    ///
    void Notification_SetIsDisplayed(msdk_LocalNotification* p_ln, msdk_s8 p_isDisplayed);

    ///
    /// Set the title of a local notification
    /// @param p_ln The local notificaton to configure
    /// @param p_title The title of the notification
    ///
    void Notification_SetTitle(msdk_LocalNotification* p_ln, const char_utf8* p_title);

    ///
	/// Android only. Set the ticker of the notification as explain <a href="http://developer.android.com/reference/android/app/Notification.Builder.html#setTicker(java.lang.CharSequence)" target="_blank">here</a>.
	/// @param p_ln The local notificaton to configure
	/// @param p_title The title of the notification
	///
	void Notification_SetTicker(msdk_LocalNotification* p_ln, const char_utf8* p_title);

    ///
    /// IOS | ANDROID
    /// Add a custom key/value pair to the local notification
    /// @param p_ln The notification to configure
    /// @param p_key A key to retreive the sotred value
    /// @param p_value The value to save
    ///
    void Notification_AddKeyValue(msdk_LocalNotification* p_ln, const char_utf8* p_key, const char_utf8* p_value);

    ///
    /// IOS | ANDROID
    /// Once everything is setup (message, sound, etc) schedule the local notification to be fire in the future
    ///	@param p_ln The local notification to be scheduled
    ///
    void Notification_ScheduleLocal(msdk_LocalNotification* p_ln);

    ///
    /// IOS | ANDROID
    ///	Return the list of scheduled local notifications
    ///	@return A list of local notification
    ///
    Array_msdk_LocalNotification* Notification_GetScheculed();

    void    Notification_ReleaseScheduled(Array_msdk_LocalNotification*);

    ///
    /// IOS | ANDROID
    /// Cancel a scheduled local notification
    /// @param p_ln The local notification to cancel
    /// @return MSDK_TRUE on success, MSDK_FALSE otherwise
    ///
    msdk_s8 Notification_CancelLocal(msdk_LocalNotification* p_ln);

    ///
    /// IOS | ANDROID
    /// Cancel all the scheduled local notifications
    ///
    void Notification_CancelAllLocals();

    const char_utf8*  Notification_GetTitle(msdk_LocalNotification* p_ln);

    const char_utf8*  Notification_GetCustomValue(msdk_LocalNotification* p_ln, const char_utf8*);

    const char_utf8*  Notification_GetDate(msdk_LocalNotification* p_ln);

    const char_utf8*  Notification_GetBody(msdk_LocalNotification* p_ln);

#ifdef MSDK_UNITY_BINDING
    MSDK_EXPORT void Notification_DefinePointer(long long callbackPointer);
#endif


#ifdef UBI_IOS
#if __OBJC__
    #include <UIKit/UILocalNotification.h>
#else
    class NSData;
    class NSError;
    class NSDictionary;
    class UILocalNotification;
#endif
    ///
    /// [iOS only] Callback to call when the register has succeeded (-(void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken)
    /// @param p_token The token passed to application:didRegisterForRemoteNotificationsWithDeviceToken:
    ///
    void Notification_OnRegisterSuccess(NSData* p_token);

    ///
    /// [iOS only] Callback to call when the register has failed(-(void)application:(UIApplication *)application didFailToRegisterForRemoteNotificationsWithError:(NSError *)error)
    /// @param p_error The error passed to application:didFailToRegisterForRemoteNotificationsWithError:
    ///
    void Notification_OnRegisterFail(NSError* p_error);

    ///
    /// [iOS only] Callback to call after the initialization of the MobileSDK in order to handle Push notifications
    /// @param p_launchOptions The launch options passed to the application:didFinishLaunchingWithOptions callback
    ///
    void Notification_OnLaunch(NSDictionary* p_launchOptions);

    ///
    /// [iOS only] Callback to call this way :
    /// -(void)application: (UIApplication *)application handleActionWithIdentifier: (NSString *)identifier forRemoteNotification:(NSDictionary *)userInfo completionHandler: (void (^)())completionHandler{
    ///    Notification_OnRemote(userInfo);
    ///    completionHandler();
    ///   }
    /// @param p_payload The dictionary passed as argument of application:didReceiveRemoteNotification
    ///
    void Notification_OnRemote(NSDictionary* p_payload);

#ifndef UBI_KIWI
    ///
    /// [iOS only] Callback to call this way :
    ///-(void)application: (UIApplication *)application handleActionWithIdentifier: (NSString *)identifier forLocalNotification:(UILocalNotification *)notification completionHandler: (void (^)())completionHandler{
    ///  Notification_OnLocal(notification);
    ///  completionHandler();
    ///  }
    /// @param p_payload The dictionary passed as argument of application:didReceiveRemoteNotification
    ///
    void Notification_OnLocal(UILocalNotification* p_notification);
#endif

#endif

#ifdef __cplusplus
}
#endif

#endif
