#ifndef MOBILE_SDK_API_GAME_SESSION_H
#define MOBILE_SDK_API_GAME_SESSION_H

#include "MobileSDK/common/Utils.h"
#include "MobileSDK/social/profile/UserProfile.h"

enum msdk_GameSessionAccess
{
    MSDK_GAMESESSION_ALLOW = 1,
    MSDK_GAMESESSION_BAN = 2
};

enum msdk_GameSessionInvitationType
{
    MSDK_INVITATION_PUSH = 1,
    MSDK_INVITATION_MAILBOX = 2
};

enum msdk_BucketType
{
    MSDK_BUCKET_OBJECT,
    MSDK_BUCKET_LOG
};

enum msdk_PortalType
{
    MSDK_ALL = 0,
    MSDK_GOOGLE,
	MSDK_AMAZON,
	MSDK_WINDOWS,
	MSDK_IOS
};

struct msdk_GameSession;


typedef struct msdk_Bucket
{
    const struct msdk_GameSession*      parent;         //!< The session containing the bucket
    const char_utf8*                    bucketId;       //!< The id of the bucket
    msdk_BucketType                     type;           //!< The type of the bucket (MSDK_BUCKET_OBJECT or MSDK_BUCKET_LOG)
    const char_utf8*                    rights;         //!< The rights of the bucket
    const char_utf8*                    cas;            //!< The cas of the bucket, to avoid corruption
    const char_utf8*                    lastModified;   //!< The last modification date
    const char_utf8*                    lastFedId;      //!< The last fedeatedid who has written in the bucket

// #ifdef MSDK_BUILD_LIB
//     msdk_BucketInternal*                internal;
// #else
    void*                               internal;
// #endif
}msdk_Bucket;

#include "MobileSDK/common/Array.inl"
DECLARE_ARRAYS(msdk_Bucket)

typedef struct msdk_Match
{
    msdk_u16				maxPlayers;		//!< The maximum player amount allowed
    msdk_u16				currentPlayers;	//!< The current amount of player
    Array_msdk_key_value	indexing;		//!< Information about the match
    const char_utf8*        extra;			//!< Extra information (or error if bad request)
    const char_utf8*        status;
}msdk_Match;

DECLARE_ARRAYS(msdk_Match)

typedef struct msdk_GameSessionInfo
{
    const char_utf8*            owner;
    const char_utf8*            created;
    const char_utf8*            expiration;
    PointerArray_char_utf8      allowed;
    PointerArray_char_utf8      joined;
    PointerArray_char_utf8      banned;
}msdk_GameSessionInfo;

typedef struct msdk_GameSession
{
    const char_utf8*        sessionId;      //!< The id of the session
    const char_utf8*        cluster;        //!< The cluster on which the session is hosted
    Array_msdk_UserInfo     users;          //!< The users associated to the session
    Array_msdk_Bucket       buckets;        //!< The buckets of the session
    msdk_u32                expirationTime; //!< The expiration time of the session
    msdk_PortalType         portal;
    msdk_Match*             match;
    msdk_GameSessionInfo*   infos;
}msdk_GameSession;

DECLARE_ARRAYS(msdk_GameSession)

/*

typedef struct msdk_MatchInfo
{
	const char_utf8*            error;



	const char_utf8*            gameSessionId;
	const char_utf8*            extra;
	const char_utf8*            matchmakingStatus;
	msdk_PortalType             portal;

	msdk_u32                    product;
	msdk_u8                     players;
	const Array_msdk_key_value* indexing;
}msdk_MatchInfo;
*/

#ifdef __cplusplus
extern "C"
{
#endif
///
/// Start a request to get the sessions of the user
///
MSDK_EXPORT void GameSession_CallGet();

///
/// Get the status of the sessions'request
/// @result The status of the request
///
MSDK_EXPORT msdk_Status GameSession_StatusGet();

///
/// Get the result of the sessions'request
/// @result The sessions of the user
///
MSDK_EXPORT Array_msdk_GameSession* GameSession_ResultGet();

///
/// Release the request used to get the sessions
///
MSDK_EXPORT void    GameSession_ReleaseGet();

///
/// Prepare a GameSession to be created on the server
/// @param p_expirationtime The duration of the session in seconds
/// @return The created session
///
MSDK_EXPORT msdk_GameSession* GameSession_Prepare(msdk_u32 p_expirationtime);

///
/// Add a friend inside a session
/// Increase the number of max Players of the Game Session.
/// @param p_session The session
/// @param p_user The friend to add to the session
///
MSDK_EXPORT void GameSession_AddFriend(msdk_GameSession* p_session, const msdk_UserInfo* p_user);

///
/// Start the creation of a session
/// @param p_session The session to create
///
MSDK_EXPORT void GameSession_CallCreate(msdk_GameSession* p_session);

///
/// Get the status of a session creation
/// @return The status of the request
///
MSDK_EXPORT msdk_Status GameSession_StatusCreate();

///
/// Get the result of a session creation
/// @result The created session
///
MSDK_EXPORT msdk_GameSession* GameSession_ResultCreate();

///
/// Release the request used to create a session
///
MSDK_EXPORT void GameSession_ReleaseCreate();


///
/// Change the right of a user inside a session (allow, ban)
/// @param p_session The session to update
/// @param p_access The access to update (MSDK_GAMESESSION_ALLOW or MSDK_GAMESESSION_BAN)
/// @param p_user The user to configure
///
MSDK_EXPORT void GameSession_CallUserAccess(msdk_GameSession* p_session, msdk_GameSessionAccess p_access, msdk_UserInfo* p_user);

///
/// Get the status of a user access request
/// The user to configure
/// @result The status of the request
///
MSDK_EXPORT msdk_Status GameSession_StatusUserAccess(msdk_UserInfo* p_user);

///
/// Get the result of a user access request
/// @param The user to configure
/// @result The result of the request
///
MSDK_EXPORT msdk_Result GameSession_ResultUserAccess(msdk_UserInfo* p_user);

///
/// Release a user access request
/// @param The user who's access has been changed
///
MSDK_EXPORT void GameSession_ReleaseUserAccess(msdk_UserInfo* p_user);

///
/// Send an invitation to the users of a session.
/// Should be called once a session has been configured to let know the users they're in the session
/// @param p_session The session
/// @param p_invitationType A mask containing the type of invitation to perform (MSDK_INVITATION_PUSH, MSDK_INVITATION_MAILBOX)
/// @param p_message The invitation message
///
MSDK_EXPORT void GameSession_CallInvite(msdk_GameSession* p_session, msdk_u32 p_invitationType, const char_utf8* p_message);

///
/// Get the status of an invitation request
/// @return The status of the request
///
MSDK_EXPORT msdk_Status GameSession_StatusInvite(msdk_GameSession* p_session);

///
/// Get the result of an invitation request
/// @return The result of the request
///
MSDK_EXPORT msdk_Result GameSession_ResultInvite(msdk_GameSession* p_session);

///
/// Release an invitation request
///
MSDK_EXPORT void GameSession_ReleaseInvite(msdk_GameSession* p_session);

///
/// Start a request to stop a session
/// @param p_session The session to stop
///
MSDK_EXPORT void GameSession_CallStop(msdk_GameSession* p_session);

///
/// Get the status of the stop session request
/// @return The status of the request
///
MSDK_EXPORT msdk_Status GameSession_StatusStop(msdk_GameSession* p_session);

///
/// Get the result of the stop session request
/// @return The result of the request
///
MSDK_EXPORT msdk_Result GameSession_ResultStop(msdk_GameSession* p_session);

///
/// Release the stop session request
///
MSDK_EXPORT void GameSession_ReleaseStop(msdk_GameSession* p_session);


///
/// Prepare a bucket to be added in a session
/// @param p_session The session to which the bucket will be added to
/// @param p_name The name of the bucket
/// @param p_type The type of bucket (MSDK_BUCKET_OBJECT or MSDK_BUCKET_LOG)
/// @return The prepared bucket
///
MSDK_EXPORT msdk_Bucket* GameSession_PrepareBucket(msdk_GameSession* p_session, const char_utf8* p_name, msdk_BucketType p_bucketType);

///
/// Add some rights to a bucket for a user
/// @param p_bucket The bucket to configure
/// @param p_federatedId The federatedId of the user to add specifics rights
/// @param p_right The right of the user
///
MSDK_EXPORT void   GameSession_BucketAddRight(msdk_Bucket* p_bucket, const char_utf8* p_federatedId, const char_utf8* p_right);

///
/// Start a request to create a prepared bucket on the server
/// @param p_bucket The bucket to create
///
MSDK_EXPORT void GameSession_CallCreateBucket(msdk_Bucket* p_bucket);

///
/// Get the status of a bucket creation request
/// @param p_bucket The bucket being created
/// @return The status of the request
///
MSDK_EXPORT msdk_Status GameSession_StatusCreateBucket(msdk_Bucket* p_bucket);

///
/// Get the result of a bucket creation request
/// @param p_bucket The bucket being created
/// @return The result of the request
///
MSDK_EXPORT msdk_Result GameSession_ResultCreateBucket(msdk_Bucket* p_bucket);

///
/// Release a bucket creation request
/// @param p_bucket The created bucket
///
MSDK_EXPORT void GameSession_ReleaseCreateBucket(msdk_Bucket* p_bucket);

///
/// Start a request to retreive the buckets associated to a session
/// @param p_session The session to process
///
MSDK_EXPORT void GameSession_CallGetBuckets(msdk_GameSession* p_session);

///
/// Get the status of a CallGetBuckets request
/// @return The status of the request
///
MSDK_EXPORT msdk_Status GameSession_StatusGetBuckets();

///
/// Get the result of a CallGetBuckets request
/// @return The result of the request
///
MSDK_EXPORT msdk_Result GameSession_ResultGetBuckets();

///
/// Release a CallGetBuckets request
///
MSDK_EXPORT void GameSession_ReleaseGetBuckets();

///
/// Sends a request to delete a bucket from a session
/// @param p_bucket The bucket to delete
///
MSDK_EXPORT void GameSession_CallDeleteBucket(msdk_Bucket* p_bucket);

///
/// Get the status of a CallDeleteBucket request
/// @param p_bucket The bucket to delete
/// @return The status of the request
///
MSDK_EXPORT msdk_Status GameSession_StatusDeleteBucket(msdk_Bucket* p_bucket);

///
/// Get the result of a CallDeleteBucket request
/// @param p_bucket The bucket to delete
/// @return The result of the request
///
///
MSDK_EXPORT msdk_Result GameSession_ResultDeleteBucket(msdk_Bucket* p_bucket);

///
/// Release the memory allocated to a CallDeleteBucket request
/// @param p_bucket The bucket to delete
///
MSDK_EXPORT void GameSession_ReleaseDeleteBucket(msdk_Bucket* p_bucket);

///
/// Update/create a key/value pair in a bucket. The change is done locally, you need to call GameSession_CallSaveBucket to synchronize the change on the server.
/// @param p_bucket The bucket to update
/// @param p_key The key
/// @param p_value The value
///
MSDK_EXPORT void GameSession_BucketUpdateValue(msdk_Bucket* p_bucket, const char_utf8* p_key, const char_utf8* p_value);

///
/// Delete a key and its associated value in a bucket. The change is done locally, you need to call GameSession_CallSaveBucket to synchronize the change on the server.
/// @param p_bucket The bucket to modify
/// @param p_key The key to delete
///
MSDK_EXPORT void GameSession_BucketDeleteKey(msdk_Bucket* p_bucket, const char_utf8* p_key);

///
/// Get a value associated to a key in a bucket
/// @param p_bucket The bucket to use
/// @param p_key The key to get the value from
/// @return The value associated to the key
///
MSDK_EXPORT const char_utf8* GameSession_BucketGetValue(msdk_Bucket* p_bucket, const char_utf8* p_key);

///
/// Send a request to save a bucket on the server
/// @param p_bucket The bucket to save
///
MSDK_EXPORT void GameSession_CallSaveBucket(msdk_Bucket* p_bucket);

///
/// Get the status of a CallSaveBucket request
/// @param p_bucket The bucket to save
/// @return The status of the request
///
MSDK_EXPORT msdk_Status GameSession_StatusSaveBucket(msdk_Bucket* p_bucket);

///
/// Get the result of a CallSaveBucket request
/// @param p_bucket The bucket to save
/// @resukt The result of the request
///
MSDK_EXPORT msdk_Result GameSession_ResultSaveBucket(msdk_Bucket* p_bucket);

///
/// Release the memory allocated for a CallSaveRequest
/// @param p_bucket The bucket to save
///
MSDK_EXPORT void GameSession_ReleaseSaveBucket(msdk_Bucket* p_bucket);

///
/// Send a request to get the object data associated to a bucket
/// @param p_bucket The bucket to get the data from
///
MSDK_EXPORT void GameSession_CallBucketObjectData(msdk_Bucket* p_bucket);

///
/// Send a request to get the log data associated to a bucket
/// @param p_bucket The bucket to get the data from
/// @param p_startingRecord The start index
/// @param p_endingRecord The last index
///
MSDK_EXPORT void GameSession_CallBucketLogData(msdk_Bucket* p_bucket, msdk_u32 p_startingRecord, msdk_u32 p_endingRecord);

///
/// Get the status of a bucket data request
/// @param p_bucket The bucket used
/// @return The status of the request
///
MSDK_EXPORT msdk_Status GameSession_StatusBucketData(msdk_Bucket* p_bucket);

///
/// Get the result of a bucket data request
/// @param p_bucket The bucket used
/// @return The result of the request
///
MSDK_EXPORT msdk_Result GameSession_ResultBucketData(msdk_Bucket* p_bucket);

///
/// Release a bucket data request
/// @param p_bucket The bucket used
///
MSDK_EXPORT void GameSession_ReleaseBucketData(msdk_Bucket* p_bucket);


    // TODO: double check Thierry's bindings for Unity


MSDK_EXPORT void GameSession_SetPortal(msdk_GameSession* p_session, msdk_PortalType p_portal);

    ///
    /// add a matchmaking set of rules/indexing inside a session
    /// @param p_session The session
    /// @param indexing json string representing a matchmaking set of rules/indexing (cf doc houston)
    /// @return The created session
    ///
MSDK_EXPORT void GameSession_AddMatchmakingIndexing(msdk_GameSession* p_session, const char_utf8 *indexing);

    ///
    /// Start a request to retrieve the list of game clusters around the worl for the current game
    ///
MSDK_EXPORT void GameSession_CallGetClusters();

    ///
    /// Get the status of a cluster request
    /// @return The status of the request
    ///
MSDK_EXPORT msdk_Status GameSession_StatusGetClusters();

    ///
    /// Get the result of a cluster request
    /// @return The list of available clusters for the game
    ///
MSDK_EXPORT const PointerArray_char_utf8* GameSession_ResultGetClusters();

    ///
    /// Release a cluster request
    ///
MSDK_EXPORT void GameSession_ReleaseGetClusters();

    ///
    /// Send a request to get a list of games available to join on a specific cluster.
    /// @param p_portal : ios, google, amazon, windows or all.
    /// @param p_limit Number of available match returned. Default to 25, maximum of 100.
    /// @param p_filter map class in the form of {"key":"value"}. Ex. {"level":"30+"}
    /// @param p_cluster The cluster to work on.
    ///
MSDK_EXPORT void GameSession_CallMatchmaking(msdk_PortalType p_portal, msdk_u8 p_limit, Array_msdk_key_value *p_filter, const char_utf8* p_cluster);

    ///
    /// Get the status of a matchmaking request
    ///
MSDK_EXPORT msdk_Status GameSession_StatusMatchmaking();

    ///
    /// Get the result of a matchmaking request
    ///
MSDK_EXPORT Array_msdk_GameSession* GameSession_ResultMatchmaking();

    ///
    /// Release a matchmaking request
    ///
MSDK_EXPORT void GameSession_ReleaseMatchmaking();

    ///
    /// Send a request to connect to a open match, retrieved from the matchmaking method.
    /// @param GameSessionId : the Game Session ID to join.
    ///
MSDK_EXPORT void GameSession_CallJoinMatch(msdk_GameSession* p_session);

    ///
    /// Get the status of a join request
    ///
MSDK_EXPORT msdk_Status GameSession_StatusJoinMatch();

    ///
    /// Get the result of a join request
    ///
MSDK_EXPORT msdk_Result GameSession_ResultJoinMatch();

    ///
    /// Release a join request
    ///
MSDK_EXPORT void GameSession_ReleaseJoinMatch();

    ///
    /// Send a request to get the info specifics to the game session.
    /// @param GameSessionId : the Game Session ID to get info.
    ///
MSDK_EXPORT void GameSession_CallSessionInfo(msdk_GameSession* p_session);

    ///
    /// Get the status of a match info request
    ///
MSDK_EXPORT msdk_Status GameSession_StatusSessionInfo(msdk_GameSession* p_session);

    ///
    /// Get the result of a match info request
    ///
MSDK_EXPORT msdk_Result GameSession_ResultSessionInfo(msdk_GameSession* p_session);

    ///
    /// Release a match info request
    ///
MSDK_EXPORT void GameSession_ReleaseSessionInfo(msdk_GameSession* p_session);

#ifdef __cplusplus
}
#endif

#endif
