#ifndef MOBILE_SDK_API_USERPROFILE_H
#define MOBILE_SDK_API_USERPROFILE_H

#include "MobileSDK/common/Utils.h"
#include "MobileSDK/social/UserInfo.h"

typedef struct msdk_GameFriend
{
    const char_utf8*       federatedId;
    Array_msdk_UserInfo    friendInfos;
}msdk_GameFriend;

#include "MobileSDK/common/Array.inl"
DECLARE_ARRAYS(msdk_GameFriend)

enum msdk_UserProfileDomain
{
    MSDK_USERPROFILE_PUBLIC,
    MSDK_USERPROFILE_PRIVATE
};

enum msdk_UserProfile_SaveGameDataResult
{
    MSDK_USERPROFILE_SAVE_GAME_DATA_ERROR,
    MSDK_USERPROFILE_SAVE_GAME_DATA_SUCCESS_LOCAL,
    MSDK_USERPROFILE_SAVE_GAME_DATA_SUCCESS_LOCAL_AND_REMOTE,
};


typedef const char_utf8* (GameDataMergeCallback)(char_utf8* saves);

typedef struct msdk_ProfileInfo
{
    const char_utf8*                federatedid;
    PointerArray_msdk_UserInfo      accounts;
}msdk_ProfileInfo;

typedef struct msdk_UserProfileWindowManager
{
    void (*DisplayAccountSelection)(msdk_ProfileInfo* p_currentprofile, msdk_ProfileInfo* p_newProfile);
    void (*DisplayMergeAccounts)(msdk_ProfileInfo* p_currentprofile, msdk_ProfileInfo* p_newProfile);
    void (*DisplayUnregisterAccount)(msdk_ProfileInfo* p_currentProfile);
}msdk_UserProfileWindowManager;

#ifdef __cplusplus
extern "C"
{
#endif

    ///
    /// Register the interface to display the merge windows
    /// @param p_manager The window manager
    ///
    MSDK_EXPORT void                    UserProfile_RegisterWindowManager(msdk_UserProfileWindowManager* p_manager);


#ifdef MSDK_UNITY_BINDING
    MSDK_EXPORT void UserProfile_DefinePointers(long long displayPointer, long long mergePointer, long long unregisterPointer, long long mergeDataPointer);
#endif

    ///
    /// Callback to call when the user choose to merge or not
    /// @p_param p_acceptMerge MSDK_TRUE if the user accepts to merge the accounts, MSDK_FALSE otherwise
    ///
    MSDK_EXPORT void                    UserProfile_OnMergeAction(msdk_s8 p_acceptMerge);

    ///
    /// Callback to call when the user has chosen an account
    /// @param p_selectedProfile The profile selected by the user
    ///
    MSDK_EXPORT void                    UserProfile_OnAccountSelected(msdk_ProfileInfo* p_selectedProfile);

    ///
    /// Callback to call when the user dismiss the window telling him he has logout
    ///
    MSDK_EXPORT void                    UserProfile_OnUnregisterAction();

    ///
    /// Check if the UserProfile has been initialized
    /// @return MSDK_TRUE if initialized, MSDK_FALSE otherwise
    ///
	MSDK_EXPORT	msdk_s8					UserProfile_IsInit();


    MSDK_EXPORT void                    UserProfile_DisableNetworkManagement();

    ///
    /// Initialize the user profile
    ///
	MSDK_EXPORT	void					UserProfile_CallInit();

    ///
    /// Get the status of the user profile initialization
    /// @return The status of the request
    ///
	MSDK_EXPORT	msdk_Status				UserProfile_StatusInit();

    ///
    /// Get the result of the user profile initialization
    /// @return The result of the request
    ///
	MSDK_EXPORT	msdk_Result				UserProfile_ResultInit();

    ///
    /// Release the initialization of the user profile
    ///
	MSDK_EXPORT	void					UserProfile_ReleaseInit();

    ///
    /// Return the ubisoft token of the current profile
    /// @return The token
    ///
    MSDK_EXPORT const char_utf8*        UserProfile_GetUbiToken();

    MSDK_EXPORT const char_utf8*        UserProfile_GetFederatedId();


    ///
    /// Register the function that will be used to perform merges between several saves of the same user.
    /// This is mandatory. Use case  :
    /// - A user plays on device A logged with Facebook
    /// - The user plays on device B without being connected
    /// - The user connect to Facebook on device B.
    /// - We need to merge the 2 saves from A and B.
    /// @param p_callback The callback
    ///
    MSDK_EXPORT void                    Userprofile_RegisterMergeCallback(GameDataMergeCallback p_callback);

    ///
    /// Update a key in the user's save
    /// @param p_key The key to update
    /// @param p_value The value to set
    /// @param p_domain The domain to save to (MSDK_USERPROFILE_PUBLIC or MSDK_USERPROFILE_PRIVATE)
    ///
    MSDK_EXPORT void                    UserProfile_UpdateGameData(const char_utf8* p_key, const char_utf8* p_value, msdk_UserProfileDomain p_domain);

    ///
    /// Delete a key in the user's save
    /// @param p_key The key to delete
    /// @param p_domain The domain to save to (MSDK_USERPROFILE_PUBLIC or MSDK_USERPROFILE_PRIVATE)
    ///
    MSDK_EXPORT void                    UserProfile_DeleteGameData(const char_utf8* p_key, msdk_UserProfileDomain p_domain);

    ///
    /// Get a key from the user's save
    /// @param p_key The key to update
    /// @param p_domain The domain to save to (MSDK_USERPROFILE_PUBLIC or MSDK_USERPROFILE_PRIVATE)
    /// @return The value associated to the key
    ///
    MSDK_EXPORT const char_utf8*        UserProfile_GetGameData(const char_utf8* p_key, msdk_UserProfileDomain p_domain);

    ///
    /// If connected to the internet, the save will be synchronized on the server
    /// If not, we save it locally and it will automatically saved to the internet the next time the UserProfile_CallGameData function is called
    MSDK_EXPORT void                    UserProfile_SaveGameData();

    ///
    /// After calling UserProfile_SaveGameData(), we attempt to save data on houston, use this function to monitor the online saving attempt.
    MSDK_EXPORT msdk_Status                    UserProfile_SaveGameDataOnlineStatus();

    ///
    /// Get the result of the UserProfile_SaveGameData() online part.
    /// @return The result of the request
    MSDK_EXPORT msdk_Result                    UserProfile_SaveGameDataOnlineResult();

    ///
    /// Start to retreive the current save of the user.
    /// If the device is connected to the internet we'll retreive the data fron there
    /// Otherwise, we'll get it locally.
    ///
    MSDK_EXPORT void                    UserProfile_CallGameData();

    ///
    /// Get the status of the save retrieving request
    /// @return The status of the request
    ///
    MSDK_EXPORT msdk_Status             UserProfile_StatusGameData();

    //
    /// Get the result of the save retrieving request ( UserProfile_CallGameData() ), returns success if we succeeded to save locally
    /// @return The result of the request
    ///
    MSDK_EXPORT msdk_Result             UserProfile_ResultGameData();

    //
    /// Get the result of the save retrieving request ( UserProfile_CallGameData() ), return success if we succeeded to upload to Houston.
    /// Wait for UserProfile_StatusGameData() to return MSDK_END
    /// @return The result of the request
    ///
    MSDK_EXPORT msdk_Result             UserProfile_ResultGameDataOnline();

    ///
    /// Release the save retrieving request
    ///
    MSDK_EXPORT void                    UserProfile_ReleaseGameData();

    ///
    /// Returns the local save in json format
    ///
    MSDK_EXPORT const char_utf8*        UserProfile_GetLocalSave();

#ifdef __cplusplus
}
#endif

#endif
