#ifndef MOBILESDKPRIVATE_COMMON_REQUESTPOOL
#define MOBILESDKPRIVATE_COMMON_REQUESTPOOL

// ==================================================================================
// Mobile SDK internal stuff
#ifdef MSDK_BUILD_LIB

	#include "MobileSDKprivate/common/Utils.h"
	#include "MobileSDKprivate/common/Logging.h"
	#include "MobileSDKprivate/common/CriticalSection.h"
	#include "MobileSDKprivate/common/Assert.h"

	namespace MobileSDKAPI
	{
		///
		/// Manage the creation and existence of requests
		/// Every class that performs requests has to use this one to create requests
		///
		template<typename T, msdk_RequestType Type>
		class RequestPool
		{
		private:
			///
			/// Definition of a private struct that represents a Request
			///
			struct RequestState
			{
				T 					result;
				///
				/// The state of the request
				///
				msdk_Status 		state;

				///
				///  The type of the request
				///
				msdk_RequestType 	type;

				///
				/// Constructor of the struct.
				/// Initialize the members
				///
				RequestState()
				{
					type = NOT_A_REQUEST;
					state = MSDK_NOT_ALLOCATED;
				};
			};

			///
			/// The id of the next request that will be created
			///
			msdk_u8 				m_maxRequests;

			///
			/// Array of RequestState that contains all the requests
			///
			RequestState* 			m_requests;

			///
			/// Critical section to allow multi threading application
			///
			CriticalSectionStruct 	m_criticalSection;
		public:
			///
			/// Constructor of the class. Initialize the pool
			/// @param  maxRequests The maximum number of simultaneous requests
			///
			RequestPool(unsigned int maxRequests = 8);

			///
			/// Destructor of the class
			///
			~RequestPool();

			///
			/// Add a request in the pool
			/// @return An s8 that is the id of the created request
			///
			msdk_s8 AddRequest();

			///
			/// Get the state of a request
			/// @param p_requestId The id of the request
			/// @return The state of the request
			///
			msdk_Status GetRequestState(const msdk_s8& p_requestId);

			///
			/// Set the state of a request
			/// @param p_requestId The id of the request
			/// @param p_newState The state to set
			///
			void SetRequestState(const msdk_s8& p_requestId, const msdk_Status& p_newState);

			///
			/// Get the type of a request
			/// @param p_requestId The id of the request
			/// @return The type of the request
			///
			msdk_RequestType GetRequestType(const msdk_s8& p_requestId);

			///
			/// Get the requests associated to a type
			/// @param p_requestType The type compared
			/// @return A vector of the requests'ids of the given type
			///
			std::vector<msdk_s8> GetRequestsByType(const msdk_RequestType& p_requestType);

			///
			/// Release a request slot
			/// @param p_requestId The id of the request to release
			///
			void ReleaseRequest(const msdk_s8& p_requestId);

			void SetRequestResult(msdk_s8 p_requestId, const T& p_result);

			const T& GetRequestResult(msdk_s8 p_requestId);

			bool IsRequestIDValid(msdk_s8 p_requestId);
		};

		#include "MobileSDKprivate/common/RequestPool.inl"
	}

#endif
// ==================================================================================

#endif //MOBILE_SDK_API_REQUEST_POOL_H
