template<typename T, msdk_RequestType Type>
RequestPool<T, Type>::RequestPool(unsigned int p_maxRequests)
{
    m_requests = static_cast<RequestState*>(msdk_Alloc(p_maxRequests * sizeof(RequestState)));
    for(msdk_u32 i = 0; i < p_maxRequests; ++i)
    {
        m_requests[i].type = NOT_A_REQUEST;
        m_requests[i].state = MSDK_NOT_ALLOCATED;
    }
    
	m_maxRequests = p_maxRequests;
	CriticalSectionInitialize(m_criticalSection);
}

template<typename T, msdk_RequestType Type>
RequestPool<T, Type>::~RequestPool()
{
	msdk_Free(m_requests);
}

template<typename T, msdk_RequestType Type>
msdk_s8 RequestPool<T, Type>::AddRequest()
{
	RequestState state;
	state.state = MSDK_IN_PROGRESS;
	state.type = Type;
	Common_Log(MSDK_LOG_VERBOSE, "[RequestPool] adding request, entering critical section");
	CriticalSectionEnter(m_criticalSection);
    Common_Log(MSDK_LOG_VERBOSE, "[RequestPool] adding request, critical section entered");
	for (unsigned int i = 0; i < m_maxRequests; ++i)
	{
		if(m_requests[i].state == MSDK_NOT_ALLOCATED)
		{
			m_requests[i] = state;
            Common_Log(MSDK_LOG_VERBOSE, "[RequestPool] request added");
			CriticalSectionLeave(m_criticalSection);
			return i;
		}
	}
    Common_Log(MSDK_LOG_VERBOSE, "[RequestPool] request not added, no free slot");
	CriticalSectionLeave(m_criticalSection);
	return -1;
}

template<typename T, msdk_RequestType Type>
msdk_Status RequestPool<T, Type>::GetRequestState(const msdk_s8& p_requestId)
{
	MSDK_ASSERT(p_requestId >= 0 && p_requestId < m_maxRequests, "RequestPool::GetRequestState invalid request Id = %d", p_requestId);
	MSDK_ASSERT(m_requests[p_requestId].type == Type, "RequestPool::GetRequestState Type mismatch: %d != %d", m_requests[p_requestId].type, Type);
	
	if(p_requestId >= 0 && p_requestId < m_maxRequests)
	{
		return m_requests[p_requestId].state;
	}
	return MSDK_ID_OUT_OF_RANGE;
}

template<typename T, msdk_RequestType Type>
void RequestPool<T, Type>::SetRequestState(const msdk_s8& p_requestId, const msdk_Status& p_newState)
{
	CriticalSectionEnter(m_criticalSection);
	if(p_requestId >= 0 && p_requestId < m_maxRequests)
		m_requests[p_requestId].state = p_newState;
	CriticalSectionLeave(m_criticalSection);
}

template<typename T, msdk_RequestType Type>
msdk_RequestType RequestPool<T, Type>::GetRequestType(const msdk_s8& p_requestId)
{
	if(p_requestId >= 0 && p_requestId < m_maxRequests)
	{
		return m_requests[p_requestId].type;
	}
	return NOT_A_REQUEST;
}

template<typename T, msdk_RequestType Type>
std::vector<msdk_s8> RequestPool<T, Type>::GetRequestsByType(const msdk_RequestType& p_requestType)
{
	std::vector<msdk_s8> requestIds;
	for(unsigned int i = 0; i < m_maxRequests; ++i)
	{
		if(m_requests[i].type == p_requestType)
			requestIds.push_back(i);
	}
	return requestIds;
}

template<typename T, msdk_RequestType Type>
void RequestPool<T, Type>::ReleaseRequest(const msdk_s8& p_requestId)
{
	// Common_Log(MSDK_VERBOSE, "Enter RequestPool::ReleaseRequest(%d)", p_requestId);
	CriticalSectionEnter(m_criticalSection);
	if(p_requestId >= 0 && p_requestId < m_maxRequests)
	{
		m_requests[p_requestId].state = MSDK_NOT_ALLOCATED;
		m_requests[p_requestId].type = NOT_A_REQUEST;
	}
	CriticalSectionLeave(m_criticalSection);
	// Common_Log(MSDK_VERBOSE, "Leave RequestPool::ReleaseRequest");
}

template<typename T, msdk_RequestType Type>
void RequestPool<T, Type>::SetRequestResult(msdk_s8 p_requestId, const T& p_result)
{
	CriticalSectionEnter(m_criticalSection);
	if(p_requestId >= 0 && p_requestId < m_maxRequests)
		m_requests[p_requestId].result = p_result;
	CriticalSectionLeave(m_criticalSection);
}

template<typename T, msdk_RequestType Type>
const T& RequestPool<T, Type>::GetRequestResult(msdk_s8 p_requestId)
{
	MSDK_ASSERT(p_requestId >= 0 && p_requestId < m_maxRequests, "RequestPool::GetRequestState invalid request Id = %d", p_requestId);
	MSDK_ASSERT(m_requests[p_requestId].type == Type, "RequestPool::GetRequestState Type mismatch: %d != %d", m_requests[p_requestId].type, Type);
	
	return m_requests[p_requestId].result;
}

template<typename T, msdk_RequestType Type>
bool RequestPool<T, Type>::IsRequestIDValid(msdk_s8 p_requestId)
{
	if(p_requestId >= 0 && p_requestId < m_maxRequests && m_requests[p_requestId].state != MSDK_NOT_ALLOCATED && m_requests[p_requestId].type != NOT_A_REQUEST)
		return true;
	else
		return false;
}